/**
 ** Vimanmaincodels.cc
 **
 ** Codels called by execution task vimanMain
 **
 ** Author: 
 ** Date: 
 **
 **/

#include <portLib.h>

#include "server/vimanHeader.h"



#include <string>
#include <iostream>
#include <fstream>
#include <sstream>
#include <stdio.h>
#include <stdlib.h>

#include "t3d/t3d.h"
#include "t3d/v3d.h"

#include <pom/pomStruct.h>
#include <viam/viamStruct.h>
#include "viam/viamtypes.h"

#include <opencv/cv.h>
#include <opencv/highgui.h>

#include <ARToolKitPlus/TrackerSingleMarkerImpl.h>
#include <ARToolKitPlus/TrackerMultiMarkerImpl.h>
#include <ARToolKitPlus/matrix.h>

void vimanDrawBox(IplImage* image, const CvMat* intrinsic_matrix, const CvMat* distortion_coeffs, CvMat* rotation_vector, CvMat* translation_vector, double size);
void vimanDrawBox2(IplImage* image, const CvMat* intrinsic_matrix, const CvMat* distortion_coeffs, CvMat* rotation_vector, CvMat* translation_vector, double dX, double dY, double dZ);
void vimanDrawFrame(IplImage* image, const CvMat* intrinsic_matrix, const CvMat* distortion_coeffs, CvMat* rotation_vector, CvMat* translation_vector, double size);
int vimanReadObjectListFile(std::string filename);
int vimanSaveObjectCfgFile(int object_index);
int vimanReadObjectCfgFile(std::string objectname, std::string filename);
int vimanFindMarkerIndex(int object_index, int marker_id, int *marker_index);
int vimanFindMarkerPoseInCameraFrame(const CvMat *intrinsic_matrix, const CvMat *dist_coeffs, CvPoint2D32f marker_corners[4], double pattern_width, CvMat *rotation_vector, CvMat *translation_vector);
int vimanCountCalibratedMarkers(int object_index);
int vimanAreAllMarkersCalibrated(int object_index);
int vimanFindObjectPoseInCameraFrame(int object_index, int marker_index, CvMat *rotation_vector, CvMat *translation_vector);
int vimanFindObjectPoseInRobotFrame(int object_index, T3D t3dCam);
int vimanFindObjectPoseInOriginFrame(int object_index, T3D t3dMainToOrigin);
void vimanResetAll();
double vimanComputeMarkerVisibility(CvPoint2D32f marker_corners[4]);
int unprojectPoint( CvMat *intrinsic_matrix,  CvPoint2D32f* image_point, double depth, CvPoint3D32f* object_point);
bool LineLineIntersect(double p1[3], double p2[3], double p3[3], double p4[3], double pa[3], double pb[3], double *mua, double *mub);
void compute_symbolic_motion(void);

//OpenCV data:
static CvCapture *CAPTURE_L = NULL;
static IplImage *IMAGE_SRC_L= NULL; //source image (no modification).
static IplImage *IMAGE_GRAY_L= NULL; //gray scale copy of the source image
static IplImage *IMAGE_GRAY_EQUALIZED_L = NULL; //gray scale image computed from IMAGE_GRAY_L with  histogram equalization
static IplImage *IMAGE_UNDIST_L= NULL; //undistorted gray scale image 
static IplImage *IMAGE_MOD_L= NULL; //modified image (image used for treatment plus information displayed over it)
CvMat *INTRINSIC_MATRIX_L= NULL;  //intrinsic matrix of the camera (filled with above calibration parameters)
CvMat *DIST_COEFFS_L= NULL;   //distortion coefficients of the camera (filled with above calibration parameters)
static double ROTATION_MATRIX_L[9]; // 3D rotation matrix from rectified to original (3x3) 
static CvFont FONT;

static IplImage *IMAGE_OUT_L = NULL;
static IplImage *IMAGE_OUT_R = NULL;
static ViamImageBank * VIMAN_BANK = NULL;
static POSTER_ID VIMAN_BANK_POSTER_ID ;
static int VIMAN_BANK_SIZE = 0;

CvMat *ROTATION_VECTOR_L[VIMAN_MAX_OBJ_NB]; 
CvMat *ROTATION_VECTOR_MARKER_L[VIMAN_MAX_OBJ_NB]; 
CvMat *TRANSLATION_VECTOR_L[VIMAN_MAX_OBJ_NB];
CvMat *ROTATION_VECTOR_R[VIMAN_MAX_OBJ_NB];   
CvMat *TRANSLATION_VECTOR_R[VIMAN_MAX_OBJ_NB];

static CvCapture *CAPTURE_R = NULL;
static IplImage *IMAGE_SRC_R= NULL; //source image (no modification).
static IplImage *IMAGE_GRAY_R= NULL; //gray scale copy of the source image
static IplImage *IMAGE_GRAY_EQUALIZED_R = NULL; //gray scale image computed from IMAGE_GRAY_L with  histogram equalization
static IplImage *IMAGE_UNDIST_R= NULL; //undistorted gray scale image 
static IplImage *IMAGE_MOD_R= NULL; //modified image (image used for treatment plus information displayed over it)
CvMat *INTRINSIC_MATRIX_R= NULL;  //intrinsic matrix of the camera (filled with above calibration parameters)
CvMat *DIST_COEFFS_R= NULL;   //distortion coefficients of the camera (filled with above calibration parameters)
static double ROTATION_MATRIX_R[9]; // 3D rotation matrix from rectified to original (3x3) 
//static  double DEPHT_CORRECTION_STEREO;

// to compute symbolic object motion
static VimanEuler PREV_OBJ_POS[VIMAN_MAX_OBJ_NB];

//ARToolkitPlus data:
ARToolKitPlus::TrackerSingleMarker *TRACKER_L= NULL;
ARToolKitPlus::TrackerSingleMarker *TRACKER_R= NULL;
static unsigned char *CAMERA_BUFFER_L= NULL;
static unsigned char *CAMERA_BUFFER_R= NULL;
class MyLogger : public ARToolKitPlus::Logger
{
  void artLog(const char* nStr)
  {  printf("%s", nStr);  }
};

//general data:
static  std::string PARAM_FILE_PATH; //contains the name of the directory containing all the data files
static size_t NB_PIXELS;  
static int IMAGE_WIDTH= 0, IMAGE_HEIGHT= 0;
static bool PRINT_INFO= false;

//if we use VIAM to get the images:
static char VIAM_CAMERA_POSTER_NAME[VIMAN_MAX_STRLEN];
static POSTER_ID VIAM_CAMERA_POSTER_ID;	/* current camera poster */
static ViamImageBank *VIAM_IMAGE;	/* current poster addr */
static int VIAM_IMAGE_NUM;
static int VIAM_CAMERA_LEFT_ID;
static int VIAM_CAMERA_RIGHT_ID;

static double VIAM_RATIO_BASELINE;

//for POM module
static POM_SENSOR_POS POM_IMAGE_POS_L;		/* current image position */
static POM_SENSOR_POS POM_IMAGE_POS_R;		/* current image position */

static  unsigned long tacq_usec;	/** time of acquisition (microseconds) */
static  unsigned long tacq_sec;	/** time of acquisition (microseconds) */
static VimanBool  CORRECT_DISTORSION;

static  T3D t3d_L_old, t3d_origin_L_old;

/*------------------------------------------------------------------------
 *
 * vimanMainEnd  --  Termination codel
 *
 * Description: 
 * 
 * Returns:    OK or ERROR
 */

STATUS
vimanMainEnd(void)
{
  vimanResetAll();

  return OK;
}

// return 0 : same, return 1:different
int vimanCompareT3d(T3D a, T3D b, double err) {
  for(int i=0; i<6; i++) {
    if(fabs(a.euler.euler[i] - b.euler.euler[i]) > err) {
      return 1;
    }
  }
  return 0;
}

/*------------------------------------------------------------------------
 *
 * vimanMainPerm  --  1st codel of permanent activity (called BEFORE other activities)
 *
 * Returns:    OK or ERROR
 */

STATUS
vimanMainPerm(int *report)
{
  int i, j, dir, shift;
  int object_index, marker_index;
  double pattern_width_L, pattern_width_R;
  char text[200], text1[200];
  double visibility;
  bool baseFrameFound= false;   
  T3D t3d_L, t3d_R, t3d_origin_L, t3d_origin_R;
  IplImage *frame= NULL;
  int	numDetected= 0;
  ARToolKitPlus::ARMarkerInfo *markers= NULL;
  CvScalar colors[4];

  colors[0]= CV_RGB(255, 0, 0);
  colors[1]= CV_RGB(0, 255, 0);
  colors[2]= CV_RGB(0, 0, 255);
  colors[3]= CV_RGB(255, 255, 0);
 
  CvPoint2D32f marker_corners_L[4];
  CvPoint2D32f marker_corners_R[4];
  CvPoint2D32f marker_corners[4];
  double pbaseLine = 0.0;

  // stereo variables
  double pmarker[4][3];
  double center[3];
  CvPoint2D32f corners[4];
  double x1, y1, z1, x, y, z;
  double tr0, tr1, tr2, tr3, tr4, tr5, tr6, tr7, tr8; // Les 9 lments de 'ROTATION_MATRIX_L' qui serviront  ``drectifier'' l'image 3D 
  int v=0;
  double mx, my;

  if(SDI_F->vimanInitIsDone != VIMAN_ENABLE) {
    *report= S_viman_VIMAN_UNINITIALIZED;
    return OK;
  }
  if(SDI_F->flagTracking==VIMAN_DISABLE ) {
    return OK;
  }
  
  // ChronoOn ( );

  switch(SDI_F->initStruct.inputStream) {
  case VIMAN_INPUT_WEBCAM:
    IMAGE_SRC_L = cvQueryFrame(CAPTURE_L); // Acquier l'image suivante
    
    cvCvtColor(IMAGE_SRC_L, IMAGE_GRAY_L, CV_BGR2GRAY);
    cvCopy(IMAGE_GRAY_L,  IMAGE_UNDIST_L, 0);
    
    for(i=0; i<NB_PIXELS; i++) {
      CAMERA_BUFFER_L[i]=  (unsigned char) IMAGE_UNDIST_L->imageData[i];
    }
    
    for(i=0; i<SDI_F->objectArray.nbObjects; i++) {
      SDI_F->objectArray.objects[i].found_L= 0;
      SDI_F->objectArray.objects[i].found_R= 0;
      
      SDI_F->objectPublicArray.objects[i].found_Left = 0;
      SDI_F->objectPublicArray.objects[i].found_Right = 0;
      SDI_F->objectPublicArray.objects[i].found_Stereo = 0;
      SDI_F->objectArray.objects[i].marker_visibility_L= 0;
	SDI_F->objectArray.objects[i].marker_visibility_R= 0;
    }
    
    
    TRACKER_L->arDetectMarker(CAMERA_BUFFER_L, 160, &markers, &numDetected);
    cvMerge( IMAGE_UNDIST_L, IMAGE_UNDIST_L, IMAGE_UNDIST_L, NULL, IMAGE_MOD_L); 
    
    //Object localization:
    //For each detected marker:
    for(i=0; i<numDetected; i++){
      if(markers[i].id!=-1){
	
	dir= markers[i].dir;
	shift= 3 - dir;     
	
	marker_corners_L[0]= cvPoint2D32f(markers[i].vertex[shift][0], markers[i].vertex[shift][1]);
	marker_corners_L[1]= cvPoint2D32f(markers[i].vertex[(1+shift)%4][0], markers[i].vertex[(1+shift)%4][1]);
	marker_corners_L[2]= cvPoint2D32f(markers[i].vertex[(2+shift)%4][0], markers[i].vertex[(2+shift)%4][1]);
	marker_corners_L[3]= cvPoint2D32f(markers[i].vertex[(3+shift)%4][0], markers[i].vertex[(3+shift)%4][1]);
	
	cvFindCornerSubPix( IMAGE_UNDIST_L,marker_corners_L, 4, cvSize(7,7), cvSize(-1,-1),
			    cvTermCriteria( CV_TERMCRIT_EPS, 0, 0.25 ) );
	
	for(j=0; j<4; j++) {
	  cvCircle(IMAGE_MOD_L, cvPoint((int) marker_corners_L[j].x, (int) marker_corners_L[j].y), 3, colors[j], 0, CV_AA, 0); 
	}
	
	if(PRINT_INFO) {
	  sprintf(text, "%d", markers[i].id);
	  cvPutText(IMAGE_MOD_L, text, cvPoint((int) markers[i].pos[0], (int) markers[i].pos[1]), &FONT, CV_RGB(255, 0, 0));
	  printf("\nmarker %d detected with %2.2f confidence\n", markers[i].id, markers[i].cf);
	  sprintf(text, "A");
	  cvPutText(IMAGE_MOD_L, text, cvPoint((int) marker_corners_L[0].x, (int) marker_corners_L[0].y), &FONT, CV_RGB(255, 0, 0));
	  sprintf(text, "B");
	  cvPutText(IMAGE_MOD_L, text, cvPoint((int) marker_corners_L[1].x, (int) marker_corners_L[1].y), &FONT, CV_RGB(255, 0, 0));
	  sprintf(text, "C");
	  cvPutText(IMAGE_MOD_L, text, cvPoint((int) marker_corners_L[2].x, (int) marker_corners_L[2].y), &FONT, CV_RGB(255, 0, 0));
	  sprintf(text, "D");
	  cvPutText(IMAGE_MOD_L, text, cvPoint((int) marker_corners_L[3].x, (int) marker_corners_L[3].y), &FONT, CV_RGB(255, 0, 0));
	}

	//For each object in the list:
	for(object_index=0; object_index<SDI_F->objectArray.nbObjects; object_index++) {
	  if(!vimanFindMarkerIndex(object_index, markers[i].id, &marker_index)) {
	    continue;
	  }
	  if(SDI_F->objectArray.objects[object_index].track==VIMAN_DISABLE) {
	    continue;
	  }
	  if(SDI_F->objectArray.objects[object_index].calibrate==VIMAN_ENABLE) {
	    continue;
	  }

	  SDI_F->objectArray.objects[object_index].cornersL[marker_index][0].x = marker_corners_L[0].x;
	  SDI_F->objectArray.objects[object_index].cornersL[marker_index][0].y = marker_corners_L[0].y;
	  SDI_F->objectArray.objects[object_index].cornersL[marker_index][1].x = marker_corners_L[1].x;
	  SDI_F->objectArray.objects[object_index].cornersL[marker_index][1].y = marker_corners_L[1].y;
	  SDI_F->objectArray.objects[object_index].cornersL[marker_index][2].x = marker_corners_L[2].x;
	  SDI_F->objectArray.objects[object_index].cornersL[marker_index][2].y = marker_corners_L[2].y;
	  SDI_F->objectArray.objects[object_index].cornersL[marker_index][3].x = marker_corners_L[3].x;
	  SDI_F->objectArray.objects[object_index].cornersL[marker_index][3].y = marker_corners_L[3].y;

	  visibility= vimanComputeMarkerVisibility(marker_corners_L);
	    
	  //if the object was already detected with another marker that offers a better visibility:
	  if(SDI_F->objectArray.objects[object_index].found_L==1 && SDI_F->objectArray.objects[object_index].marker_visibility_L>visibility) {
	    continue;
	  }

	  SDI_F->objectArray.objects[object_index].marker_visibility_L= visibility;
	    
	  pattern_width_L= SDI_F->objectArray.objects[object_index].patternWidths[marker_index];
	    
	  SDI_F->objectArray.objects[object_index].patternWidth_maxVisible_L = pattern_width_L;	
	  SDI_F->objectArray.objects[object_index].markerIndex_maxVisible_L = marker_index;
	    
	  vimanFindMarkerPoseInCameraFrame(INTRINSIC_MATRIX_L, DIST_COEFFS_L, marker_corners_L, pattern_width_L, ROTATION_VECTOR_L[object_index], TRANSLATION_VECTOR_L[object_index]);
	    
	  cvmSet(ROTATION_VECTOR_MARKER_L[object_index], 0, 0, cvmGet( ROTATION_VECTOR_L[object_index], 0, 0));
	  cvmSet(ROTATION_VECTOR_MARKER_L[object_index], 1, 0, cvmGet( ROTATION_VECTOR_L[object_index], 1, 0));
	  cvmSet(ROTATION_VECTOR_MARKER_L[object_index], 2, 0, cvmGet( ROTATION_VECTOR_L[object_index], 2, 0));
	    
	  if(vimanFindObjectPoseInCameraFrame(object_index, marker_index, ROTATION_VECTOR_L[object_index], TRANSLATION_VECTOR_L[object_index])) {
	    sprintf(text, "%s", SDI_F->objectArray.objects[object_index].name);
	    cvPutText(IMAGE_MOD_L, text, cvPoint((int) markers[i].pos[0], (int) markers[i].pos[1]+30), &FONT, CV_RGB(0, 255, 0));
	      
	    //vimanDrawBox2(IMAGE_MOD_L, INTRINSIC_MATRIX_L, DIST_COEFFS_L, ROTATION_VECTOR, TRANSLATION_VECTOR, 78, 20, 107);
	    //vimanDrawFrame(IMAGE_MOD_L, INTRINSIC_MATRIX_L, DIST_COEFFS_L, ROTATION_VECTOR_L[object_index], TRANSLATION_VECTOR_L[object_index], pattern_width_L);
	      
	    SDI_F->objectArray.objects[object_index].found_L= 1;
	    SDI_F->objectPublicArray.objects[object_index].found_Left = 1;     
	      
	  }
	  continue;
	}
      }
    }
      
    //======================================================================//
    //   Display the frame of each object and the position of the Trash     //
    //======================================================================//
      
    //For each object in the list:
    for(object_index=0; object_index<SDI_F->objectArray.nbObjects; object_index++) {
      sprintf(SDI_F->objectPublicArray.objects[object_index].name, "%s", SDI_F->objectArray.objects[object_index].name ); 
	
	
      //    strcpy(&(SDI_F->objectPublicArray.objects[object_index].name), SDI_F->objectArray.objects[object_index].name);
	
      if(strcmp(SDI_F->objectArray.objects[object_index].name, "TRASHBIN")==0) {
	sprintf(text, "Trash X= %2.3f Y=%2.3f Z=%2.3f", SDI_F->objectPublicArray.objects[object_index].eulerOrigin.x,
		  SDI_F->objectPublicArray.objects[object_index].eulerOrigin.y, 
		  SDI_F->objectPublicArray.objects[object_index].eulerOrigin.z);
	  sprintf(text1, "     Y= %2.3f P=%2.3f R=%2.3f", SDI_F->objectPublicArray.objects[object_index].eulerOrigin.yaw,
		  SDI_F->objectPublicArray.objects[object_index].eulerOrigin.pitch, 
		  SDI_F->objectPublicArray.objects[object_index].eulerOrigin.roll);
	} 
	
	if( SDI_F->objectPublicArray.objects[object_index].found_Stereo == 1) {
	  // Draw frame in the left image
	  vimanDrawFrame(IMAGE_MOD_L, INTRINSIC_MATRIX_L, DIST_COEFFS_L, ROTATION_VECTOR_L[object_index],
			 TRANSLATION_VECTOR_L[object_index], SDI_F->objectArray.objects[object_index].patternWidth_maxVisible_L );
	  if(strcmp(SDI_F->objectArray.objects[object_index].name, "TRASHBIN")==0) {
	    cvPutText(IMAGE_MOD_L, text, cvPoint( 0, IMAGE_HEIGHT-80), &FONT, CV_RGB(255, 255, 0));
	    cvPutText(IMAGE_MOD_L, text1, cvPoint( 0, IMAGE_HEIGHT-40), &FONT, CV_RGB(255, 255, 0));
	  }
	  
	} else if( SDI_F->objectPublicArray.objects[object_index].found_Left == 1) {
	  // Draw frame in the left image
	  vimanDrawFrame(IMAGE_MOD_L, INTRINSIC_MATRIX_L, DIST_COEFFS_L, ROTATION_VECTOR_L[object_index],
			 TRANSLATION_VECTOR_L[object_index], SDI_F->objectArray.objects[object_index].patternWidth_maxVisible_L );
	  if(strcmp(SDI_F->objectArray.objects[object_index].name, "TRASHBIN")==0) {
	    cvPutText(IMAGE_MOD_L, text, cvPoint( 0, IMAGE_HEIGHT-80), &FONT, CV_RGB(255, 0, 0));
	  }
	}
      }
      cvShowImage("VIMAN_Left", IMAGE_MOD_L);
      cvWaitKey(3);
      break;
    case VIMAN_INPUT_VIAM:
      if (posterTake(VIAM_CAMERA_POSTER_ID, POSTER_READ) != OK )
	{
	  h2perror((char*)"vimanInitMain: viman posterTake");
	  *report = S_viman_CANNOT_READ_POSTER;
	  return ETHER;
	}
      VIAM_IMAGE = (ViamImageBank *)posterAddr(VIAM_CAMERA_POSTER_ID);
      if(VIAM_IMAGE == NULL) 
     {
       posterGive(VIAM_CAMERA_POSTER_ID);
       *report = S_viman_CANNOT_READ_POSTER;
       return ETHER;
     }
      if(VIAM_IMAGE->nImages <= 0)
	{
	  printf("vimanInitMain : There is no image acquired!\n");
	  *report = S_viman_CANNOT_READ_POSTER;
	  return ETHER;        
	}
      
      if(VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].width!=IMAGE_WIDTH || VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].height!=IMAGE_HEIGHT)
	{
	  *report= S_viman_VIDEO_MODE_INCONSISTENCY;
	  return ERROR;
	}
      
      
      memcpy(IMAGE_SRC_L->imageData, VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].data+VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].dataOffset,VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].imageSize);

      /* save the time of acquisition */
      tacq_usec = VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].tacq_usec;
      tacq_sec =  VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].tacq_sec;

 
      if(SDI_F->initStruct.stereo == VIMAN_ENABLE) {
	memcpy(IMAGE_SRC_R->imageData, VIAM_IMAGE->image[VIAM_CAMERA_RIGHT_ID].data+VIAM_IMAGE->image[VIAM_CAMERA_RIGHT_ID].dataOffset,VIAM_IMAGE->image[VIAM_CAMERA_RIGHT_ID].imageSize);
      }
      pbaseLine = VIAM_IMAGE->calibration.pbaseline;
      
      POM_IMAGE_POS_L = VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].pos;

      if(SDI_F->initStruct.stereo == VIMAN_ENABLE) {
	POM_IMAGE_POS_R = VIAM_IMAGE->image[VIAM_CAMERA_RIGHT_ID].pos;  
      } 

      posterGive(VIAM_CAMERA_POSTER_ID);

     //left
      // Get transformation to 'main' frame 
      t3d_L = t3dBryanIdentity;
      t3d_L.euler.euler[0] = POM_IMAGE_POS_L.sensorToMain.euler.yaw;
      t3d_L.euler.euler[1] = POM_IMAGE_POS_L.sensorToMain.euler.pitch;
      t3d_L.euler.euler[2] = POM_IMAGE_POS_L.sensorToMain.euler.roll;
      t3d_L.euler.euler[3] = POM_IMAGE_POS_L.sensorToMain.euler.x;
      t3d_L.euler.euler[4] = POM_IMAGE_POS_L.sensorToMain.euler.y;
      t3d_L.euler.euler[5] = POM_IMAGE_POS_L.sensorToMain.euler.z;

   
      // fill in variances 
      t3d_L.euler.v[0] = POM_IMAGE_POS_L.sensorToMain.var.voo;
      t3d_L.euler.v[1] = POM_IMAGE_POS_L.sensorToMain.var.vop;
      t3d_L.euler.v[2] = POM_IMAGE_POS_L.sensorToMain.var.vpp;
      t3d_L.euler.v[3] = POM_IMAGE_POS_L.sensorToMain.var.vor;
      t3d_L.euler.v[4] = POM_IMAGE_POS_L.sensorToMain.var.vpr;
      t3d_L.euler.v[5] = POM_IMAGE_POS_L.sensorToMain.var.vrr;
      t3d_L.euler.v[6] = POM_IMAGE_POS_L.sensorToMain.var.vox;
      t3d_L.euler.v[7] = POM_IMAGE_POS_L.sensorToMain.var.vpx;
      t3d_L.euler.v[8] = POM_IMAGE_POS_L.sensorToMain.var.vrx;
      t3d_L.euler.v[9] = POM_IMAGE_POS_L.sensorToMain.var.vxx;
      t3d_L.euler.v[10]= POM_IMAGE_POS_L.sensorToMain.var.voy;
      t3d_L.euler.v[11]= POM_IMAGE_POS_L.sensorToMain.var.vpy;
      t3d_L.euler.v[12]= POM_IMAGE_POS_L.sensorToMain.var.vry;
      t3d_L.euler.v[13]= POM_IMAGE_POS_L.sensorToMain.var.vxy;
      t3d_L.euler.v[14]= POM_IMAGE_POS_L.sensorToMain.var.vyy;
      t3d_L.euler.v[15]= POM_IMAGE_POS_L.sensorToMain.var.voz;
      t3d_L.euler.v[16]= POM_IMAGE_POS_L.sensorToMain.var.vpz;
      t3d_L.euler.v[17]= POM_IMAGE_POS_L.sensorToMain.var.vrz;
      t3d_L.euler.v[18]= POM_IMAGE_POS_L.sensorToMain.var.vxz;
      t3d_L.euler.v[19]= POM_IMAGE_POS_L.sensorToMain.var.vyz;
      t3d_L.euler.v[20]= POM_IMAGE_POS_L.sensorToMain.var.vzz;


      /* Check if camera has move3d since last period */
      if(vimanCompareT3d(t3d_L, t3d_L_old, 0.01)!=0 || SDI_F->flagUserTracking == VIMAN_DISABLE) {
	printf("viman camera moves reset object detection \n");
	t3d_L_old = t3d_L;

	for(i=0; i<SDI_F->objectArray.nbObjects; i++) {
	  SDI_F->objectArray.objects[i].found_L= 0;
	  SDI_F->objectArray.objects[i].found_R= 0;	  
	  SDI_F->objectPublicArray.objects[i].found_Left = 0;
	  SDI_F->objectPublicArray.objects[i].found_Right = 0;
	  SDI_F->objectPublicArray.objects[i].found_Stereo = 0;
	  SDI_F->objectArray.objects[i].marker_visibility_L= 0;
	  SDI_F->objectArray.objects[i].marker_visibility_R= 0;
	}

	return OK;
      }
      t3d_L_old = t3d_L;


      t3dConvertTo(T3D_MATRIX, &t3d_L);

      SDI_F->CAMERA_LEFT_ROB_POSE.nx = t3d_L.matrix[0] ;
      SDI_F->CAMERA_LEFT_ROB_POSE.ny = t3d_L.matrix[4] ;
      SDI_F->CAMERA_LEFT_ROB_POSE.nz = t3d_L.matrix[8] ;    			  
      SDI_F->CAMERA_LEFT_ROB_POSE.ox = t3d_L.matrix[1] ;
      SDI_F->CAMERA_LEFT_ROB_POSE.oy = t3d_L.matrix[5] ;
      SDI_F->CAMERA_LEFT_ROB_POSE.oz = t3d_L.matrix[9] ;
      SDI_F->CAMERA_LEFT_ROB_POSE.ax = t3d_L.matrix[2] ;
      SDI_F->CAMERA_LEFT_ROB_POSE.ay = t3d_L.matrix[6] ;
      SDI_F->CAMERA_LEFT_ROB_POSE.az = t3d_L.matrix[10];
      SDI_F->CAMERA_LEFT_ROB_POSE.px = t3d_L.matrix[3] ;
      SDI_F->CAMERA_LEFT_ROB_POSE.py = t3d_L.matrix[7] ;
      SDI_F->CAMERA_LEFT_ROB_POSE.pz = t3d_L.matrix[11]; 


      // Get transformation to 'origin' frame 
      t3d_origin_L = t3dBryanIdentity;
      t3d_origin_L.euler.euler[0] = POM_IMAGE_POS_L.mainToOrigin.euler.yaw;
      t3d_origin_L.euler.euler[1] = POM_IMAGE_POS_L.mainToOrigin.euler.pitch;
      t3d_origin_L.euler.euler[2] = POM_IMAGE_POS_L.mainToOrigin.euler.roll;
      t3d_origin_L.euler.euler[3] = POM_IMAGE_POS_L.mainToOrigin.euler.x;
      t3d_origin_L.euler.euler[4] = POM_IMAGE_POS_L.mainToOrigin.euler.y;
      t3d_origin_L.euler.euler[5] = POM_IMAGE_POS_L.mainToOrigin.euler.z;

      /* Check if camera has move3d since last period */
      if(vimanCompareT3d(t3d_origin_L, t3d_origin_L_old, 0.01)!=0) {
	printf("viman robot moves reset object detection \n");
	t3d_origin_L_old = t3d_origin_L;

	for(i=0; i<SDI_F->objectArray.nbObjects; i++) {
	  SDI_F->objectArray.objects[i].found_L= 0;
	  SDI_F->objectArray.objects[i].found_R= 0;	  
	  SDI_F->objectPublicArray.objects[i].found_Left = 0;
	  SDI_F->objectPublicArray.objects[i].found_Right = 0;
	  SDI_F->objectPublicArray.objects[i].found_Stereo = 0;
	  SDI_F->objectArray.objects[i].marker_visibility_L= 0;
	  SDI_F->objectArray.objects[i].marker_visibility_R= 0;
	}

	return OK;
      }
      t3d_origin_L_old = t3d_origin_L;

      t3dConvertTo(T3D_MATRIX, &t3d_origin_L);



      if(SDI_F->initStruct.stereo == VIMAN_ENABLE) {
      //right
      // Get transformation to 'main' frame 
      t3d_R = t3dBryanIdentity;
      t3d_R.euler.euler[0] = POM_IMAGE_POS_R.sensorToMain.euler.yaw;
      t3d_R.euler.euler[1] = POM_IMAGE_POS_R.sensorToMain.euler.pitch;
      t3d_R.euler.euler[2] = POM_IMAGE_POS_R.sensorToMain.euler.roll;
      t3d_R.euler.euler[3] = POM_IMAGE_POS_R.sensorToMain.euler.x;
      t3d_R.euler.euler[4] = POM_IMAGE_POS_R.sensorToMain.euler.y;
      t3d_R.euler.euler[5] = POM_IMAGE_POS_R.sensorToMain.euler.z;
	 
      // fill in variances 
      t3d_R.euler.v[0] = POM_IMAGE_POS_R.sensorToMain.var.voo;
      t3d_R.euler.v[1] = POM_IMAGE_POS_R.sensorToMain.var.vop;
      t3d_R.euler.v[2] = POM_IMAGE_POS_R.sensorToMain.var.vpp;
      t3d_R.euler.v[3] = POM_IMAGE_POS_R.sensorToMain.var.vor;
      t3d_R.euler.v[4] = POM_IMAGE_POS_R.sensorToMain.var.vpr;
      t3d_R.euler.v[5] = POM_IMAGE_POS_R.sensorToMain.var.vrr;
      t3d_R.euler.v[6] = POM_IMAGE_POS_R.sensorToMain.var.vox;
      t3d_R.euler.v[7] = POM_IMAGE_POS_R.sensorToMain.var.vpx;
      t3d_R.euler.v[8] = POM_IMAGE_POS_R.sensorToMain.var.vrx;
      t3d_R.euler.v[9] = POM_IMAGE_POS_R.sensorToMain.var.vxx;
      t3d_R.euler.v[10]= POM_IMAGE_POS_R.sensorToMain.var.voy;
      t3d_R.euler.v[11]= POM_IMAGE_POS_R.sensorToMain.var.vpy;
      t3d_R.euler.v[12]= POM_IMAGE_POS_R.sensorToMain.var.vry;
      t3d_R.euler.v[13]= POM_IMAGE_POS_R.sensorToMain.var.vxy;
      t3d_R.euler.v[14]= POM_IMAGE_POS_R.sensorToMain.var.vyy;
      t3d_R.euler.v[15]= POM_IMAGE_POS_R.sensorToMain.var.voz;
      t3d_R.euler.v[16]= POM_IMAGE_POS_R.sensorToMain.var.vpz;
      t3d_R.euler.v[17]= POM_IMAGE_POS_R.sensorToMain.var.vrz;
      t3d_R.euler.v[18]= POM_IMAGE_POS_R.sensorToMain.var.vxz;
      t3d_R.euler.v[19]= POM_IMAGE_POS_R.sensorToMain.var.vyz;
      t3d_R.euler.v[20]= POM_IMAGE_POS_R.sensorToMain.var.vzz;

      t3dConvertTo(T3D_MATRIX, &t3d_R);

      // Get transformation to 'origin' frame 
      t3d_origin_R = t3dBryanIdentity;
      t3d_origin_R.euler.euler[0] = POM_IMAGE_POS_R.mainToOrigin.euler.yaw;
      t3d_origin_R.euler.euler[1] = POM_IMAGE_POS_R.mainToOrigin.euler.pitch;
      t3d_origin_R.euler.euler[2] = POM_IMAGE_POS_R.mainToOrigin.euler.roll;
      t3d_origin_R.euler.euler[3] = POM_IMAGE_POS_R.mainToOrigin.euler.x;
      t3d_origin_R.euler.euler[4] = POM_IMAGE_POS_R.mainToOrigin.euler.y;
      t3d_origin_R.euler.euler[5] = POM_IMAGE_POS_R.mainToOrigin.euler.z;
      t3dConvertTo(T3D_MATRIX, &t3d_origin_R);

      SDI_F->CAMERA_RIGHT_ROB_POSE.nx = t3d_R.matrix[0] ;
      SDI_F->CAMERA_RIGHT_ROB_POSE.ny = t3d_R.matrix[4] ;
      SDI_F->CAMERA_RIGHT_ROB_POSE.nz = t3d_R.matrix[8] ;    			  
      SDI_F->CAMERA_RIGHT_ROB_POSE.ox = t3d_R.matrix[1] ;
      SDI_F->CAMERA_RIGHT_ROB_POSE.oy = t3d_R.matrix[5] ;
      SDI_F->CAMERA_RIGHT_ROB_POSE.oz = t3d_R.matrix[9] ;
      SDI_F->CAMERA_RIGHT_ROB_POSE.ax = t3d_R.matrix[2] ;
      SDI_F->CAMERA_RIGHT_ROB_POSE.ay = t3d_R.matrix[6] ;
      SDI_F->CAMERA_RIGHT_ROB_POSE.az = t3d_R.matrix[10];
      SDI_F->CAMERA_RIGHT_ROB_POSE.px = t3d_R.matrix[3] ;
      SDI_F->CAMERA_RIGHT_ROB_POSE.py = t3d_R.matrix[7] ;
      SDI_F->CAMERA_RIGHT_ROB_POSE.pz = t3d_R.matrix[11]; 


      }





      if(SDI_F->initStruct.color == VIMAN_ENABLE) {
	cvCvtColor(IMAGE_SRC_L, IMAGE_GRAY_L, CV_BGR2GRAY);
      } else {
	cvCopy(IMAGE_SRC_L,  IMAGE_GRAY_L, 0);
      }
      if(SDI_F->initStruct.stereo == VIMAN_ENABLE) {
	if(SDI_F->initStruct.color == VIMAN_ENABLE) {
	  cvCvtColor(IMAGE_SRC_R, IMAGE_GRAY_R, CV_BGR2GRAY);
	} else {
	  cvCopy(IMAGE_SRC_R,  IMAGE_GRAY_R, 0);
	}
      }



      if(CORRECT_DISTORSION == VIMAN_ENABLE) { 
	cvEqualizeHist(IMAGE_GRAY_L, IMAGE_GRAY_EQUALIZED_L);
	cvUndistort2(IMAGE_GRAY_EQUALIZED_L, IMAGE_UNDIST_L, INTRINSIC_MATRIX_L, DIST_COEFFS_L);
      } else {
	cvEqualizeHist(IMAGE_GRAY_L, IMAGE_GRAY_EQUALIZED_L);	
	cvCopy(IMAGE_GRAY_EQUALIZED_L,  IMAGE_UNDIST_L, 0);
      }
      if(SDI_F->initStruct.stereo == VIMAN_ENABLE) {
	if(CORRECT_DISTORSION == VIMAN_ENABLE) { 
	  cvEqualizeHist(IMAGE_GRAY_R, IMAGE_GRAY_EQUALIZED_R);
	  cvUndistort2(IMAGE_GRAY_R, IMAGE_UNDIST_R, INTRINSIC_MATRIX_R, DIST_COEFFS_R);
	} else {
	  cvEqualizeHist(IMAGE_GRAY_R, IMAGE_GRAY_EQUALIZED_R);	  
	  cvCopy(IMAGE_GRAY_EQUALIZED_R,  IMAGE_UNDIST_R, 0);
	}
      }

      for(i=0; i<NB_PIXELS; i++) {
	  CAMERA_BUFFER_L[i]=  (unsigned char) IMAGE_UNDIST_L->imageData[i];
	  if(SDI_F->initStruct.stereo == VIMAN_ENABLE) {
	    CAMERA_BUFFER_R[i]=  (unsigned char) IMAGE_UNDIST_R->imageData[i];
	  }
      }
  
      for(i=0; i<SDI_F->objectArray.nbObjects; i++) {
	SDI_F->objectArray.objects[i].found_L= 0;
	SDI_F->objectArray.objects[i].found_R= 0;
	
	
	SDI_F->objectPublicArray.objects[i].found_Left = 0;
	SDI_F->objectPublicArray.objects[i].found_Right = 0;
	SDI_F->objectPublicArray.objects[i].found_Stereo = 0;
	SDI_F->objectArray.objects[i].marker_visibility_L= 0;
	SDI_F->objectArray.objects[i].marker_visibility_R= 0;
	}

      //======================================================================//
      //                 Detect Objects in Image LEFT                          //
      //======================================================================//

      TRACKER_L->arDetectMarker(CAMERA_BUFFER_L, 160, &markers, &numDetected);
      cvMerge( IMAGE_UNDIST_L, IMAGE_UNDIST_L, IMAGE_UNDIST_L, NULL, IMAGE_MOD_L); 

      for(object_index=0; object_index<SDI_F->objectArray.nbObjects; object_index++) {
	SDI_F->objectArray.objects[object_index].found_L = 0;
	for(v = 0; v<  SDI_F->objectArray.objects[object_index].nbMarkers; v++) {
	  SDI_F->objectArray.objects[object_index].markerFoundL[v] = 0;
	  SDI_F->objectArray.objects[object_index].markerVisibilityL[v] = 0.0;
	}
      }
  
      //Object localization:
      //For each detected marker:
      for(i=0; i<numDetected; i++){
	if(markers[i].id!=-1){
      
	  dir= markers[i].dir;
	  shift= 3 - dir;     

	  marker_corners_L[0]= cvPoint2D32f(markers[i].vertex[shift][0], markers[i].vertex[shift][1]);
	  marker_corners_L[1]= cvPoint2D32f(markers[i].vertex[(1+shift)%4][0], markers[i].vertex[(1+shift)%4][1]);
	  marker_corners_L[2]= cvPoint2D32f(markers[i].vertex[(2+shift)%4][0], markers[i].vertex[(2+shift)%4][1]);
	  marker_corners_L[3]= cvPoint2D32f(markers[i].vertex[(3+shift)%4][0], markers[i].vertex[(3+shift)%4][1]);

	  cvFindCornerSubPix( IMAGE_UNDIST_L,marker_corners_L, 4, cvSize(7,7), cvSize(-1,-1), cvTermCriteria( CV_TERMCRIT_EPS, 0, 0.25 ) );
	  for(j=0; j<4; j++) {
	    cvCircle(IMAGE_MOD_L, cvPoint((int) marker_corners_L[j].x, (int) marker_corners_L[j].y), 3, colors[j], 0, CV_AA, 0); 
	  }

//	  if(PRINT_INFO) {
//	    sprintf(text, "%d", markers[i].id);
//	    cvPutText(IMAGE_MOD_L, text, cvPoint((int) markers[i].pos[0], (int) markers[i].pos[1]), &FONT, CV_RGB(255, 0, 0));
//	    printf("\nmarker %d detected with %2.2f confidence\n", markers[i].id, markers[i].cf);
//	    sprintf(text, "A");
//	    cvPutText(IMAGE_MOD_L, text, cvPoint((int) marker_corners_L[0].x, (int) marker_corners_L[0].y), &FONT, CV_RGB(255, 0, 0));
//	    sprintf(text, "B");
//	    cvPutText(IMAGE_MOD_L, text, cvPoint((int) marker_corners_L[1].x, (int) marker_corners_L[1].y), &FONT, CV_RGB(255, 0, 0));
//	    sprintf(text, "C");
//	    cvPutText(IMAGE_MOD_L, text, cvPoint((int) marker_corners_L[2].x, (int) marker_corners_L[2].y), &FONT, CV_RGB(255, 0, 0));
//	    sprintf(text, "D");
//	    cvPutText(IMAGE_MOD_L, text, cvPoint((int) marker_corners_L[3].x, (int) marker_corners_L[3].y), &FONT, CV_RGB(255, 0, 0));
//	  }

	  //For each object in the list:
	  for(object_index=0; object_index<SDI_F->objectArray.nbObjects; object_index++) {

	    if(!vimanFindMarkerIndex(object_index, markers[i].id, &marker_index)) {
	      continue;
	    }
	    if(SDI_F->objectArray.objects[object_index].track==VIMAN_DISABLE) {
	      continue;
	    }
	    if(SDI_F->objectArray.objects[object_index].calibrate==VIMAN_ENABLE) {
	      continue;
	    }
	    
	    SDI_F->objectArray.objects[object_index].cornersL[marker_index][0].x = marker_corners_L[0].x;
	    SDI_F->objectArray.objects[object_index].cornersL[marker_index][0].y = marker_corners_L[0].y;
	    SDI_F->objectArray.objects[object_index].cornersL[marker_index][1].x = marker_corners_L[1].x;
	    SDI_F->objectArray.objects[object_index].cornersL[marker_index][1].y = marker_corners_L[1].y;
	    SDI_F->objectArray.objects[object_index].cornersL[marker_index][2].x = marker_corners_L[2].x;
	    SDI_F->objectArray.objects[object_index].cornersL[marker_index][2].y = marker_corners_L[2].y;
	    SDI_F->objectArray.objects[object_index].cornersL[marker_index][3].x = marker_corners_L[3].x;
	    SDI_F->objectArray.objects[object_index].cornersL[marker_index][3].y = marker_corners_L[3].y;

	    //<<<<<<< HEAD:codels/vimanMainCodels.cc
	    if(vimanFindObjectPoseInRobotFrame(object_index, t3d_L)) {
	      // compute pose in origin frame

	      SDI_F->objectArray.objects[object_index].markerFoundL[marker_index] = 1;
	      SDI_F->objectArray.objects[object_index].markerVisibilityL[marker_index] = vimanComputeMarkerVisibility(marker_corners_L);
	      SDI_F->objectArray.objects[object_index].found_L = 1;
	      SDI_F->objectPublicArray.objects[object_index].found_Left = 1; 
	      SDI_F->objectPublicArray.objects[object_index].found_Right = 0;
	      SDI_F->objectPublicArray.objects[object_index].found_Stereo = 0;
	      vimanFindObjectPoseInOriginFrame(object_index,t3d_origin_L);
	      SDI_F->objectArray.objects[object_index].tacq_usec = tacq_usec ;
	      SDI_F->objectPublicArray.objects[object_index].tacq_usec = tacq_usec ;
	      SDI_F->objectArray.objects[object_index].tacq_sec = tacq_sec ;
	      SDI_F->objectPublicArray.objects[object_index].tacq_sec = tacq_sec ;
	    }
	    continue;
	  }
	}
      }
      
      if(SDI_F->initStruct.stereo == VIMAN_ENABLE) {
      //======================================================================//
      //                 Detect objects in Image RIGHT                         //
      //======================================================================//

      TRACKER_R->arDetectMarker(CAMERA_BUFFER_R, 160, &markers, &numDetected);  
      cvMerge( IMAGE_UNDIST_R, IMAGE_UNDIST_R, IMAGE_UNDIST_R, NULL, IMAGE_MOD_R); 
     
      for(object_index=0; object_index<SDI_F->objectArray.nbObjects; object_index++) {
	SDI_F->objectArray.objects[object_index].found_R = 0;
	for(v = 0; v<  SDI_F->objectArray.objects[object_index].nbMarkers; v++) {
	  SDI_F->objectArray.objects[object_index].markerFoundR[v] = 0;
	  SDI_F->objectArray.objects[object_index].markerVisibilityR[v] = 0.0;
	}
      }

      //Object localization:
      //For each detected marker:
      for(i=0; i<numDetected; i++) {
	if(markers[i].id!=-1) {
      
	  dir= markers[i].dir;
	  shift= 3 - dir;     

	  marker_corners_R[0]= cvPoint2D32f(markers[i].vertex[shift][0], markers[i].vertex[shift][1]);
	  marker_corners_R[1]= cvPoint2D32f(markers[i].vertex[(1+shift)%4][0], markers[i].vertex[(1+shift)%4][1]);
	  marker_corners_R[2]= cvPoint2D32f(markers[i].vertex[(2+shift)%4][0], markers[i].vertex[(2+shift)%4][1]);
	  marker_corners_R[3]= cvPoint2D32f(markers[i].vertex[(3+shift)%4][0], markers[i].vertex[(3+shift)%4][1]);

	  cvFindCornerSubPix( IMAGE_UNDIST_R,marker_corners_R, 4, cvSize(7,7), cvSize(-1,-1),
			      cvTermCriteria( CV_TERMCRIT_EPS, 0, 0.25 ) );

	  for(j=0; j<4; j++) {
	    cvCircle(IMAGE_MOD_R, cvPoint((int) marker_corners_R[j].x, (int) marker_corners_R[j].y), 3, colors[j], 0, CV_AA, 0); 
	  }
      
//	  if(PRINT_INFO) {
//	    sprintf(text, "%d", markers[i].id);
//	    cvPutText(IMAGE_MOD_R, text, cvPoint((int) markers[i].pos[0], (int) markers[i].pos[1]), &FONT, CV_RGB(255, 0, 0));
//	    printf("\nmarker %d detected with %2.2f confidence\n", markers[i].id, markers[i].cf);
//	    sprintf(text, "A");
//	    cvPutText(IMAGE_MOD_R, text, cvPoint((int) marker_corners_R[0].x, (int) marker_corners_R[0].y), &FONT, CV_RGB(255, 0, 0));
//	    sprintf(text, "B");
//	    cvPutText(IMAGE_MOD_R, text, cvPoint((int) marker_corners_R[1].x, (int) marker_corners_R[1].y), &FONT, CV_RGB(255, 0, 0));
//	    sprintf(text, "C");
//	    cvPutText(IMAGE_MOD_R, text, cvPoint((int) marker_corners_R[2].x, (int) marker_corners_R[2].y), &FONT, CV_RGB(255, 0, 0));
//	    sprintf(text, "D");
//	    cvPutText(IMAGE_MOD_R, text, cvPoint((int) marker_corners_R[3].x, (int) marker_corners_R[3].y), &FONT, CV_RGB(255, 0, 0));
//	  }

	  //For each object in the list:
	  for(object_index=0; object_index<SDI_F->objectArray.nbObjects; object_index++) {

	    if(!vimanFindMarkerIndex(object_index, markers[i].id, &marker_index)) {
	      continue;
	    }
	    if(SDI_F->objectArray.objects[object_index].track==VIMAN_DISABLE) {
	      continue;
	    }
	    if(SDI_F->objectArray.objects[object_index].calibrate==VIMAN_ENABLE) {
	      continue;
	      }
	    
	    SDI_F->objectArray.objects[object_index].cornersR[marker_index][0].x = marker_corners_R[0].x;
	    SDI_F->objectArray.objects[object_index].cornersR[marker_index][0].y = marker_corners_R[0].y;
	    SDI_F->objectArray.objects[object_index].cornersR[marker_index][1].x = marker_corners_R[1].x;
	    SDI_F->objectArray.objects[object_index].cornersR[marker_index][1].y = marker_corners_R[1].y;
	    SDI_F->objectArray.objects[object_index].cornersR[marker_index][2].x = marker_corners_R[2].x;
	    SDI_F->objectArray.objects[object_index].cornersR[marker_index][2].y = marker_corners_R[2].y;
	    SDI_F->objectArray.objects[object_index].cornersR[marker_index][3].x = marker_corners_R[3].x;
	    SDI_F->objectArray.objects[object_index].cornersR[marker_index][3].y = marker_corners_R[3].y;

	    SDI_F->objectArray.objects[object_index].markerVisibilityR[marker_index] = vimanComputeMarkerVisibility(marker_corners_R);
	    SDI_F->objectArray.objects[object_index].markerFoundR[marker_index] = 1;;
	    SDI_F->objectArray.objects[object_index].found_R = 1;
	  }
	}
      }
      }
      
      //======================================================================//
      //                       Compute 3D pose                                //
      //======================================================================//

      
      
      //For each object in the list:
      for(object_index=0; object_index<SDI_F->objectArray.nbObjects; object_index++) { // For object
	
	SDI_F->objectPublicArray.objects[object_index].found_Stereo = 0;   
	SDI_F->objectPublicArray.objects[object_index].found_Left = 0; 
	SDI_F->objectPublicArray.objects[object_index].found_Right = 0;


	// STEREO 
	if( SDI_F->objectArray.objects[object_index].found_L== 1 &&   SDI_F->objectArray.objects[object_index].found_R== 1) {
	  // Find the marker to be used
	  SDI_F->objectPublicArray.objects[object_index].found_Stereo = 1;  
	  SDI_F->objectPublicArray.objects[object_index].found_Left = 1; 
	  SDI_F->objectPublicArray.objects[object_index].found_Right = 1;

	  for(v = 0; v<  SDI_F->objectArray.objects[object_index].nbMarkers; v++) {
	    if(SDI_F->objectArray.objects[object_index].markerFoundL[v] == 1 && SDI_F->objectArray.objects[object_index].markerFoundR[v] == 1) {
	    
	    
	      pattern_width_L= SDI_F->objectArray.objects[object_index].patternWidths[v];
	      pattern_width_R = pattern_width_L;
	      SDI_F->objectArray.objects[object_index].patternWidth_maxVisible_L = pattern_width_L;
	      SDI_F->objectArray.objects[object_index].patternWidth_maxVisible_R = pattern_width_L;    
	      SDI_F->objectArray.objects[object_index].markerIndex_maxVisible_L = v;
	      SDI_F->objectArray.objects[object_index].markerIndex_maxVisible_R = v;

	      marker_corners_L[0].x = SDI_F->objectArray.objects[object_index].cornersL[v][0].x;
	      marker_corners_L[0].y = SDI_F->objectArray.objects[object_index].cornersL[v][0].y;
	      marker_corners_L[1].x = SDI_F->objectArray.objects[object_index].cornersL[v][1].x;
	      marker_corners_L[1].y = SDI_F->objectArray.objects[object_index].cornersL[v][1].y;
	      marker_corners_L[2].x = SDI_F->objectArray.objects[object_index].cornersL[v][2].x;
	      marker_corners_L[2].y = SDI_F->objectArray.objects[object_index].cornersL[v][2].y;
	      marker_corners_L[3].x = SDI_F->objectArray.objects[object_index].cornersL[v][3].x;
	      marker_corners_L[3].y = SDI_F->objectArray.objects[object_index].cornersL[v][3].y;

	      vimanFindMarkerPoseInCameraFrame(INTRINSIC_MATRIX_L, DIST_COEFFS_L, marker_corners_L, 
					       pattern_width_L, ROTATION_VECTOR_L[object_index], TRANSLATION_VECTOR_L[object_index]);
	      cvmSet(ROTATION_VECTOR_MARKER_L[object_index], 0, 0, cvmGet( ROTATION_VECTOR_L[object_index], 0, 0));
	      cvmSet(ROTATION_VECTOR_MARKER_L[object_index], 1, 0, cvmGet( ROTATION_VECTOR_L[object_index], 1, 0));
	      cvmSet(ROTATION_VECTOR_MARKER_L[object_index], 2, 0, cvmGet( ROTATION_VECTOR_L[object_index], 2, 0));


	      marker_corners_R[0].x = SDI_F->objectArray.objects[object_index].cornersR[v][0].x;
	      marker_corners_R[0].y = SDI_F->objectArray.objects[object_index].cornersR[v][0].y;
	      marker_corners_R[1].x = SDI_F->objectArray.objects[object_index].cornersR[v][1].x;
	      marker_corners_R[1].y = SDI_F->objectArray.objects[object_index].cornersR[v][1].y;
	      marker_corners_R[2].x = SDI_F->objectArray.objects[object_index].cornersR[v][2].x;
	      marker_corners_R[2].y = SDI_F->objectArray.objects[object_index].cornersR[v][2].y;
	      marker_corners_R[3].x = SDI_F->objectArray.objects[object_index].cornersR[v][3].x;
	      marker_corners_R[3].y = SDI_F->objectArray.objects[object_index].cornersR[v][3].y;
	      
	      vimanFindMarkerPoseInCameraFrame(INTRINSIC_MATRIX_R, DIST_COEFFS_R, marker_corners_R, pattern_width_R, ROTATION_VECTOR_R[object_index], TRANSLATION_VECTOR_R[object_index]);
	      
	      if(vimanFindObjectPoseInCameraFrame(object_index, v, ROTATION_VECTOR_L[object_index], TRANSLATION_VECTOR_L[object_index])) {
		sprintf(text, "%s", SDI_F->objectArray.objects[object_index].name);
		mx = SDI_F->objectArray.objects[object_index].cornersL[v][0].x;
		my = SDI_F->objectArray.objects[object_index].cornersL[v][0].y;
		cvPutText(IMAGE_MOD_L, text, cvPoint((int) mx, (int) my + 30), &FONT, CV_RGB(0, 255, 0));
	      }

	      if(vimanFindObjectPoseInCameraFrame(object_index, v, ROTATION_VECTOR_R[object_index], TRANSLATION_VECTOR_R[object_index])) {
		sprintf(text, "%s", SDI_F->objectArray.objects[object_index].name);
		mx = SDI_F->objectArray.objects[object_index].cornersR[v][0].x;
		my = SDI_F->objectArray.objects[object_index].cornersR[v][0].y;
		cvPutText(IMAGE_MOD_R, text, cvPoint((int)  mx, (int) my+30), &FONT, CV_RGB(0, 255, 0));
	      }
	     
	      for(i=0; i<4; i++) {
		marker_corners_L[i].x =  SDI_F->objectArray.objects[object_index].cornersL[SDI_F->objectArray.objects[object_index].markerIndex_maxVisible_L][i].x;
		marker_corners_L[i].y =  SDI_F->objectArray.objects[object_index].cornersL[SDI_F->objectArray.objects[object_index].markerIndex_maxVisible_L][i].y;
		marker_corners_R[i].x =  SDI_F->objectArray.objects[object_index].cornersR[SDI_F->objectArray.objects[object_index].markerIndex_maxVisible_L][i].x;
		marker_corners_R[i].y =  SDI_F->objectArray.objects[object_index].cornersR[SDI_F->objectArray.objects[object_index].markerIndex_maxVisible_L][i].y;
	      }
	      
	      // compute object pose in robot frame
	      for(i=0; i<4; i++) {
		z1 =  (pbaseLine*VIAM_RATIO_BASELINE) / (marker_corners_L[i].x - marker_corners_R[i].x ) ;
		x1 = z1 * ( marker_corners_L[i].x - cvmGet(INTRINSIC_MATRIX_L, 0, 2)) / cvmGet(INTRINSIC_MATRIX_L, 0, 0) ;
		y1 = z1 * (marker_corners_L[i].y - cvmGet(INTRINSIC_MATRIX_L, 1, 2) ) / cvmGet(INTRINSIC_MATRIX_L, 1, 1) ;
		tr0 = ROTATION_MATRIX_L[0];
		tr1 = ROTATION_MATRIX_L[1];
		tr2 = ROTATION_MATRIX_L[2];
		tr3 = ROTATION_MATRIX_L[3];
		tr4 = ROTATION_MATRIX_L[4];
		tr5 = ROTATION_MATRIX_L[5];
		tr6 = ROTATION_MATRIX_L[6];
		tr7 = ROTATION_MATRIX_L[7];
		tr8 = ROTATION_MATRIX_L[8];
		/* ...et on applique la rotation. */
		x = (tr0 * x1) + (tr3 * y1) + (tr6 * z1);
		y = (tr1 * x1) + (tr4 * y1) + (tr7 * z1);
		z = (tr2 * x1) + (tr5 * y1) + (tr8 * z1);
		/* save the 3D point of marker in mm */
		pmarker[i][0]= x * 1000;
		pmarker[i][1]= y * 1000;
		pmarker[i][2]= z * 1000;
	      }
	      
	      center[0]= (pmarker[0][0] + pmarker[1][0] + pmarker[2][0] + pmarker[3][0] ) / 4.0;
	      center[1]= (pmarker[0][1] + pmarker[1][1] + pmarker[2][1] + pmarker[3][1] ) / 4.0;
	      center[2]= (pmarker[0][2] + pmarker[1][2] + pmarker[2][2] + pmarker[3][2] ) / 4.0;
   
	      cvmSet(TRANSLATION_VECTOR_L[object_index], 0, 0, center[0] + SDI_F->corImageFrame.dx ); 
	      cvmSet(TRANSLATION_VECTOR_L[object_index], 1, 0, center[1] + SDI_F->corImageFrame.dy );  
	      cvmSet(TRANSLATION_VECTOR_L[object_index], 2, 0, center[2] + SDI_F->corImageFrame.dz ); 
	      
	      //TODO compute ROTATION_VECTOR
     
	      //compute object pose according to the pose of the marker
	      if(vimanFindObjectPoseInCameraFrame(object_index, v, ROTATION_VECTOR_MARKER_L[object_index], TRANSLATION_VECTOR_L[object_index])) {
		// Position in robot frame by stereo vision with the left image        
		vimanFindObjectPoseInRobotFrame(object_index, t3d_L);
		// compute pose in origin frame
		SDI_F->objectPublicArray.objects[object_index].found_Stereo = 1;     
		vimanFindObjectPoseInOriginFrame(object_index,t3d_origin_L);    
	      }
	      break;
	    } // end if markerFoundL[v] == 1  && markerFoundR[v] == 1

	    // if we are here it is because the maker saw in each camera is not the same
	  SDI_F->objectPublicArray.objects[object_index].found_Stereo = 0;  


	  } // end for nbMarkers
	} // Fin stereo
        if ( SDI_F->objectPublicArray.objects[object_index].found_Stereo == 0 && SDI_F->objectArray.objects[object_index].found_L== 1 ) {
	  // LEFT
	  visibility = 0.0;
	  
	  for(v = 0; v<  SDI_F->objectArray.objects[object_index].nbMarkers; v++) {
	    //if the object was already detected with another marker that offers a better visibility:
	    if( SDI_F->objectArray.objects[object_index].markerVisibilityL[v] > visibility) {
		  
	      pattern_width_L= SDI_F->objectArray.objects[object_index].patternWidths[v];
 
	      SDI_F->objectArray.objects[object_index].patternWidth_maxVisible_L = pattern_width_L;	
	      SDI_F->objectArray.objects[object_index].markerIndex_maxVisible_L = v;

	     marker_corners_L[0].x = SDI_F->objectArray.objects[object_index].cornersL[v][0].x;
	     marker_corners_L[0].y = SDI_F->objectArray.objects[object_index].cornersL[v][0].y;
	     marker_corners_L[1].x = SDI_F->objectArray.objects[object_index].cornersL[v][1].x;
	     marker_corners_L[1].y = SDI_F->objectArray.objects[object_index].cornersL[v][1].y;
	     marker_corners_L[2].x = SDI_F->objectArray.objects[object_index].cornersL[v][2].x;
	     marker_corners_L[2].y = SDI_F->objectArray.objects[object_index].cornersL[v][2].y;
	     marker_corners_L[3].x = SDI_F->objectArray.objects[object_index].cornersL[v][3].x;
	     marker_corners_L[3].y = SDI_F->objectArray.objects[object_index].cornersL[v][3].y;

	    }
	  }

	  v = SDI_F->objectArray.objects[object_index].markerIndex_maxVisible_L;

	  vimanFindMarkerPoseInCameraFrame(INTRINSIC_MATRIX_L, DIST_COEFFS_L, marker_corners_L, pattern_width_L, ROTATION_VECTOR_L[object_index], TRANSLATION_VECTOR_L[object_index]);

	  cvmSet(TRANSLATION_VECTOR_L[object_index], 0, 0,( cvmGet( TRANSLATION_VECTOR_L[object_index], 0, 0) + SDI_F->corImageFrame.dx) ); 
	  cvmSet(TRANSLATION_VECTOR_L[object_index], 1, 0,( cvmGet( TRANSLATION_VECTOR_L[object_index], 1, 0) + SDI_F->corImageFrame.dy) ); 	  
	  cvmSet(TRANSLATION_VECTOR_L[object_index], 2, 0,( cvmGet( TRANSLATION_VECTOR_L[object_index], 2, 0) + SDI_F->corImageFrame.dz) ); 

	  cvmSet(ROTATION_VECTOR_MARKER_L[object_index], 0, 0, cvmGet( ROTATION_VECTOR_L[object_index], 0, 0));
	  cvmSet(ROTATION_VECTOR_MARKER_L[object_index], 1, 0, cvmGet( ROTATION_VECTOR_L[object_index], 1, 0));
	  cvmSet(ROTATION_VECTOR_MARKER_L[object_index], 2, 0, cvmGet( ROTATION_VECTOR_L[object_index], 2, 0));
	  

	  if(vimanFindObjectPoseInCameraFrame(object_index, v, ROTATION_VECTOR_L[object_index], TRANSLATION_VECTOR_L[object_index])) {
	    sprintf(text, "%s", SDI_F->objectArray.objects[object_index].name);
	    mx = SDI_F->objectArray.objects[object_index].cornersL[v][0].x;
	    my = SDI_F->objectArray.objects[object_index].cornersL[v][0].y;
	    cvPutText(IMAGE_MOD_L, text, cvPoint((int) mx , (int) my+30), &FONT, CV_RGB(0, 255, 0));
	    
	    //vimanDrawBox2(IMAGE_MOD_L, INTRINSIC_MATRIX_L, DIST_COEFFS_L, ROTATION_VECTOR, TRANSLATION_VECTOR, 78, 20, 107);
	    //vimanDrawFrame(IMAGE_MOD_L, INTRINSIC_MATRIX_L, DIST_COEFFS_L, ROTATION_VECTOR_L[object_index], TRANSLATION_VECTOR_L[object_index], pattern_width_L);
	    
	    SDI_F->objectArray.objects[object_index].found_L= 1;
	    if(strcmp(SDI_F->objectArray.objects[object_index].name, "baseFrame")==0) {
	      baseFrameFound= true; 
	    }            
	    
	    if(vimanFindObjectPoseInRobotFrame(object_index, t3d_L)) {
		  // compute pose in origin frame
	      SDI_F->objectPublicArray.objects[object_index].found_Left = 1; 
	      vimanFindObjectPoseInOriginFrame(object_index,t3d_origin_L);
	    }
	  }
	}
	else if ( SDI_F->objectPublicArray.objects[object_index].found_Stereo == 0 && SDI_F->objectArray.objects[object_index].found_R == 1 ) {
	  // RIGHT
	  visibility = 0.0;

	  for(v = 0; v<  SDI_F->objectArray.objects[object_index].nbMarkers; v++) {
	    //if the object was already detected with another marker that offers a better visibility:
	    if( SDI_F->objectArray.objects[object_index].markerVisibilityR[v] > visibility) {
		  
	      pattern_width_R= SDI_F->objectArray.objects[object_index].patternWidths[v];
 
	      SDI_F->objectArray.objects[object_index].patternWidth_maxVisible_R = pattern_width_R;	
	      SDI_F->objectArray.objects[object_index].markerIndex_maxVisible_R = v;

	     marker_corners_R[0].x = SDI_F->objectArray.objects[object_index].cornersR[v][0].x;
	     marker_corners_R[0].y = SDI_F->objectArray.objects[object_index].cornersR[v][0].y;
	     marker_corners_R[1].x = SDI_F->objectArray.objects[object_index].cornersR[v][1].x;
	     marker_corners_R[1].y = SDI_F->objectArray.objects[object_index].cornersR[v][1].y;
	     marker_corners_R[2].x = SDI_F->objectArray.objects[object_index].cornersR[v][2].x;
	     marker_corners_R[2].y = SDI_F->objectArray.objects[object_index].cornersR[v][2].y;
	     marker_corners_R[3].x = SDI_F->objectArray.objects[object_index].cornersR[v][3].x;
	     marker_corners_R[3].y = SDI_F->objectArray.objects[object_index].cornersR[v][3].y;
	    }
	  }

	  v = SDI_F->objectArray.objects[object_index].markerIndex_maxVisible_L;
	  vimanFindMarkerPoseInCameraFrame(INTRINSIC_MATRIX_R, DIST_COEFFS_R, marker_corners_R, pattern_width_R, ROTATION_VECTOR_R[object_index], TRANSLATION_VECTOR_R[object_index]);

	  cvmSet(TRANSLATION_VECTOR_R[object_index], 0, 0,( cvmGet( TRANSLATION_VECTOR_R[object_index], 0, 0) + SDI_F->corImageFrame.dx) ); 
	  cvmSet(TRANSLATION_VECTOR_R[object_index], 1, 0,( cvmGet( TRANSLATION_VECTOR_R[object_index], 1, 0) + SDI_F->corImageFrame.dy) ); 	
	  cvmSet(TRANSLATION_VECTOR_R[object_index], 2, 0,( cvmGet( TRANSLATION_VECTOR_R[object_index], 2, 0) + SDI_F->corImageFrame.dz) ); 
	  
	  if(vimanFindObjectPoseInCameraFrame(object_index, v, ROTATION_VECTOR_R[object_index], TRANSLATION_VECTOR_R[object_index])) {
	    sprintf(text, "%s", SDI_F->objectArray.objects[object_index].name);
	    mx = SDI_F->objectArray.objects[object_index].cornersR[v][0].x;
	    my = SDI_F->objectArray.objects[object_index].cornersR[v][0].y;
	    cvPutText(IMAGE_MOD_R, text, cvPoint((int) mx, (int) my+30), &FONT, CV_RGB(0, 255, 0));
	    
	    //vimanDrawBox2(IMAGE_MOD_R, INTRINSIC_MATRIX_R, DIST_COEFFS_R, ROTATION_VECTOR, TRANSLATION_VECTOR, 78, 20, 107);
	    //vimanDrawFrame(IMAGE_MOD_R, INTRINSIC_MATRIX_R, DIST_COEFFS_R, ROTATION_VECTOR_R[object_index], TRANSLATION_VECTOR_R[object_index], pattern_width_R);
	    
	    SDI_F->objectArray.objects[object_index].found_R= 1;
	    if(strcmp(SDI_F->objectArray.objects[object_index].name, "baseFrame")==0) {
	      baseFrameFound= true; 
	    }          
	    
	    if(vimanFindObjectPoseInRobotFrame(object_index, t3d_R)) {
	      // compute pose in origin frame
	      SDI_F->objectPublicArray.objects[object_index].found_Right = 1;     
	      // If the object is already found in the left image we don't compute the origin pose for the right image
	      if(SDI_F->objectPublicArray.objects[object_index].found_Left == 0) {
		vimanFindObjectPoseInOriginFrame(object_index,t3d_origin_R);
	      }
	    }
	  }
	} // end RIGHT
      } // end for object
      
      //======================================================================//
      //   Display the frame of each object and the position of the Trash     //
      //======================================================================//

      //For each object in the list:
      for(object_index=0; object_index<SDI_F->objectArray.nbObjects; object_index++) {

	sprintf(SDI_F->objectPublicArray.objects[object_index].name, "%s", SDI_F->objectArray.objects[object_index].name ); 

	if( SDI_F->objectPublicArray.objects[object_index].found_Stereo == 1) {
	  vimanDrawFrame(IMAGE_MOD_L, INTRINSIC_MATRIX_L, DIST_COEFFS_L, ROTATION_VECTOR_L[object_index],
			 TRANSLATION_VECTOR_L[object_index], SDI_F->objectArray.objects[object_index].patternWidth_maxVisible_L );
	  vimanDrawFrame(IMAGE_MOD_R, INTRINSIC_MATRIX_R, DIST_COEFFS_R, ROTATION_VECTOR_R[object_index],
			 TRANSLATION_VECTOR_R[object_index], SDI_F->objectArray.objects[object_index].patternWidth_maxVisible_R );
	
	} else {
	  if( SDI_F->objectPublicArray.objects[object_index].found_Left == 1) { // Draw frame in the left image
	    vimanDrawFrame(IMAGE_MOD_L, INTRINSIC_MATRIX_L, DIST_COEFFS_L, ROTATION_VECTOR_L[object_index],
			 TRANSLATION_VECTOR_L[object_index], SDI_F->objectArray.objects[object_index].patternWidth_maxVisible_L );
	  }
	  if( SDI_F->objectPublicArray.objects[object_index].found_Right == 1) { // Draw frame in the right image
	    vimanDrawFrame(IMAGE_MOD_R, INTRINSIC_MATRIX_R, DIST_COEFFS_R, ROTATION_VECTOR_R[object_index],
			 TRANSLATION_VECTOR_R[object_index], SDI_F->objectArray.objects[object_index].patternWidth_maxVisible_R );
	  }
	}
      }
  
      cvShowImage("VIMAN_Left", IMAGE_MOD_L);
      if(SDI_F->initStruct.stereo == VIMAN_ENABLE) {
	cvShowImage("VIMAN_Right", IMAGE_MOD_R);
      }
      cvWaitKey(3);
 
      break;
  default:
    *report= S_viman_NO_VIDEO_MODE_IS_DEFINED;
    return OK;
    break;
}

  if ( SDI_F->initStruct.inputStream == VIMAN_INPUT_WEBCAM ||
       SDI_F->initStruct.inputStream == VIMAN_INPUT_VIAM ) {
    compute_symbolic_motion();
  }




  cvResize(IMAGE_MOD_L, IMAGE_OUT_L, CV_INTER_LINEAR);

  memcpy(VIMAN_BANK->image[0].data + VIMAN_BANK->image[0].dataOffset, 
	 IMAGE_OUT_L->imageData, IMAGE_OUT_L->imageSize);


  if(SDI_F->initStruct.stereo == VIMAN_ENABLE) {
    cvResize(IMAGE_MOD_R, IMAGE_OUT_R, CV_INTER_LINEAR);

    memcpy(VIMAN_BANK->image[1].data + VIMAN_BANK->image[1].dataOffset, 
	   IMAGE_OUT_R->imageData, IMAGE_OUT_R->imageSize);
  }

   
  
  //writing to poster_
  if( posterWrite(VIMAN_BANK_POSTER_ID, 0, VIMAN_BANK, VIMAN_BANK_SIZE) == ERROR )
    h2perror("Write error");


  //  ChronoPrint ((const char*)"chrono viman" );
  //  ChronoOff ( );
 
  return OK;
}


/*------------------------------------------------------------------------
/ compute_symbolic_motion: computes the symbolic motion for the objects.
/------------------------------------------------------------------------*/
void compute_symbolic_motion()
{
  int obj_idx;
  double prev_x, prev_y, prev_z, x, y, z, dist;
  double prev_yaw, prev_pitch, prev_roll, yaw, pitch, roll, diff_yaw, diff_pitch, diff_roll;

  for ( obj_idx = 0; obj_idx < SDI_F->objectArray.nbObjects; obj_idx++ ) {
    prev_x = PREV_OBJ_POS[obj_idx].x;
    prev_y = PREV_OBJ_POS[obj_idx].y;
    prev_z = PREV_OBJ_POS[obj_idx].z;

    x = SDI_F->objectPublicArray.objects[obj_idx].eulerOrigin.x;
    y = SDI_F->objectPublicArray.objects[obj_idx].eulerOrigin.y;
    z = SDI_F->objectPublicArray.objects[obj_idx].eulerOrigin.z;

    dist = sqrt(pow(prev_x-x,2) + pow(prev_y-y,2) + pow(prev_z-z,2));

    prev_yaw = PREV_OBJ_POS[obj_idx].yaw;
    prev_pitch = PREV_OBJ_POS[obj_idx].pitch;
    prev_roll = PREV_OBJ_POS[obj_idx].roll;

    yaw = SDI_F->objectPublicArray.objects[obj_idx].eulerOrigin.yaw;
    pitch = SDI_F->objectPublicArray.objects[obj_idx].eulerOrigin.pitch;
    roll = SDI_F->objectPublicArray.objects[obj_idx].eulerOrigin.roll;

    diff_yaw = (prev_yaw-yaw > M_PI)? fabs(prev_yaw-M_PI*2):prev_yaw-yaw; 
    diff_pitch = (prev_pitch-pitch > M_PI)? fabs(prev_pitch-M_PI*2):prev_pitch-pitch; 
    diff_roll = (prev_roll-roll > M_PI)? fabs(prev_roll-M_PI*2):prev_roll-roll; 

    if ( dist > VIMAN_DIST_THR || 
	 diff_yaw > VIMAN_ANGLE_THR ||
	 diff_pitch > VIMAN_ANGLE_THR ||
	 diff_roll > VIMAN_ANGLE_THR ) {     
      SDI_F->objectPublicArray.objects[obj_idx].motion = VIMAN_MOVING;
      PREV_OBJ_POS[obj_idx].x = x;
      PREV_OBJ_POS[obj_idx].y = y;
      PREV_OBJ_POS[obj_idx].z = z;
      PREV_OBJ_POS[obj_idx].yaw = yaw;
      PREV_OBJ_POS[obj_idx].pitch = pitch;
      PREV_OBJ_POS[obj_idx].roll = roll;
    }
    else {
      SDI_F->objectPublicArray.objects[obj_idx].motion = VIMAN_STATIC;
    }
  }
}


/*------------------------------------------------------------------------
 * Init
 *
 * Description: 
 *
 * Reports:      OK
 *              S_viman_VIMAN_UNINITIALIZED
 *              S_viman_NO_VIDEO_MODE_IS_DEFINED
 *              S_viman_CANNOT_OPEN_VIDEO_STREAM
 *              S_viman_POSTER_NOT_FOUND
 *              S_viman_CANNOT_READ_POSTER
 */

/* vimanInitMain  -  codel EXEC of Init
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
vimanInitMain(VimanInitStr *initStruct, int *report)
{
  if(SDI_F->vimanInitIsDone==VIMAN_ENABLE)
  {
    vimanResetAll();
  }

  int i, n, xsize, ysize, undist_iterations;
  char _s[32];
  char header[128];
  double kc[6];
  IplImage *frame= NULL;
  std::string fileName, objectName;
  FILE *file= NULL;
  std::ofstream fileCal;
 
  //Calibration parameters for webcam (initialized by the init function):
  double FOCAL_LENGTHS[2]= {0.0, 0.0}; double PRINCIPLE_POINT[2]= {0.0, 0.0};
  double DISTORTION[5]= {0.0, 0.0, 0.0, 0.0, 0.0};
  std::string fileNameWebCam="QuickCam640x480.cal" ;

  SDI_F->initStruct.inputStream = initStruct->inputStream;
  SDI_F->flagDisplay = initStruct->display;
  SDI_F->flagTracking= VIMAN_DISABLE;
  SDI_F->flagUserTracking= VIMAN_ENABLE;
  CORRECT_DISTORSION = VIMAN_DISABLE;
  //  CORRECT_DISTORSION = initStruct->correctDistortion;
  for(int i=0; i<VIMAN_MAX_OBJ_NB; i++)
    {	
      SDI_F->objectArray.objects[i].thetaMatRob.nx = 1;
      SDI_F->objectArray.objects[i].thetaMatRob.ny = 0;
      SDI_F->objectArray.objects[i].thetaMatRob.nz = 0;
  					  
      SDI_F->objectArray.objects[i].thetaMatRob.ox = 0;
      SDI_F->objectArray.objects[i].thetaMatRob.oy = 1;
      SDI_F->objectArray.objects[i].thetaMatRob.oz = 0;
  					  
      SDI_F->objectArray.objects[i].thetaMatRob.ax = 0;
      SDI_F->objectArray.objects[i].thetaMatRob.ay = 0;
      SDI_F->objectArray.objects[i].thetaMatRob.az = 1;
  					  
      SDI_F->objectArray.objects[i].thetaMatRob.px = 0;
      SDI_F->objectArray.objects[i].thetaMatRob.py = 0;
      SDI_F->objectArray.objects[i].thetaMatRob.pz = 0;  
      SDI_F->objectArray.objects[i].tacq_sec = -1 ;
      SDI_F->objectPublicArray.objects[i].tacq_usec = -1;

      SDI_F->objectPublicArray.objects[i].motion = VIMAN_NONE;
      PREV_OBJ_POS[i].x = PREV_OBJ_POS[i].y = PREV_OBJ_POS[i].z = 0.0;
      PREV_OBJ_POS[i].yaw = PREV_OBJ_POS[i].pitch = PREV_OBJ_POS[i].roll = 0.0;
    }

  PARAM_FILE_PATH.clear();
  PARAM_FILE_PATH += initStruct->filePath;

  printf("file path %s\n", PARAM_FILE_PATH.c_str());

  //initialize OpenCV FONT, parameters are hscale, vscale, shear, thickness, line_type
  cvInitFont( &FONT, CV_FONT_HERSHEY_SIMPLEX, 1.0, 1.0, 0, 2, 8);

  //Open the video stream:
  switch(initStruct->inputStream)
    {
    case VIMAN_NO_INPUT:
      *report= S_viman_NO_VIDEO_MODE_IS_DEFINED;
      return ETHER;
      break;
    case VIMAN_INPUT_WEBCAM:
      if(SDI_F->flagDisplay==VIMAN_ENABLE) {
	cvNamedWindow("VIMAN_Left", CV_WINDOW_AUTOSIZE);
      }

      PARAM_FILE_PATH.clear();
      PARAM_FILE_PATH += initStruct->filePath;
      
      fileName.clear();
      fileName +=  PARAM_FILE_PATH;
      fileName += "/cal/";
      fileName += "QuickCam640x480.cal";

      //Read the file:
      file = fopen(fileName.c_str(), "r");
      if(file==NULL) {
	printf("File %s was not found.\n", fileName.c_str());
	SDI_F->flagDisplay= VIMAN_DISABLE;
	return ETHER;
      }
   
      n= fscanf(file, "%s %d %d %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %d", header, &xsize, &ysize, &PRINCIPLE_POINT[0], &PRINCIPLE_POINT[1], &FOCAL_LENGTHS[0], &FOCAL_LENGTHS[1], &kc[0], &kc[1], &kc[2], &kc[3], &kc[4], &kc[5], &undist_iterations); 
      if( (n!=14) || ferror(file) ) {
	printf("The content of %s is in a bad format.\n", fileName.c_str());
	SDI_F->flagDisplay= VIMAN_DISABLE;
	return ETHER;
      }
   
      CAPTURE_L = cvCaptureFromCAM(0);
      if (CAPTURE_L==NULL) {
	printf("cannot open the webcam stream\n");
	return  ETHER;
      }
      cvSetCaptureProperty( CAPTURE_L, CV_CAP_PROP_FRAME_WIDTH , xsize );

      cvSetCaptureProperty( CAPTURE_L, CV_CAP_PROP_FRAME_HEIGHT , ysize );
      /*acquisition of an image from webcam */
      IMAGE_SRC_L  = cvQueryFrame(CAPTURE_L); // Acquier l'image suivante
      printf("acquisition webcam ok\n");

      IMAGE_WIDTH = cvGetSize(IMAGE_SRC_L).width;
      IMAGE_HEIGHT= cvGetSize(IMAGE_SRC_L).height;

      NB_PIXELS = (size_t)(IMAGE_WIDTH*IMAGE_HEIGHT);
      CAMERA_BUFFER_L = new unsigned char[NB_PIXELS];
      
      IMAGE_GRAY_L  = cvCreateImage(cvSize(IMAGE_WIDTH, IMAGE_HEIGHT), 8, 1);
      IMAGE_GRAY_EQUALIZED_L  = cvCreateImage(cvSize(IMAGE_WIDTH, IMAGE_HEIGHT), 8, 1);
      IMAGE_UNDIST_L= cvCreateImage(cvSize(IMAGE_WIDTH, IMAGE_HEIGHT), 8, 1);
      IMAGE_MOD_L   = cvCreateImage(cvSize(IMAGE_WIDTH, IMAGE_HEIGHT), 8, 3);
      cvCvtColor(IMAGE_SRC_L, IMAGE_GRAY_L, CV_BGR2GRAY);
      cvEqualizeHist(IMAGE_GRAY_L, IMAGE_GRAY_EQUALIZED_L);
      cvCopy(IMAGE_GRAY_EQUALIZED_L, IMAGE_UNDIST_L, 0);
      cvMerge( IMAGE_UNDIST_L, IMAGE_UNDIST_L, IMAGE_UNDIST_L, NULL, IMAGE_MOD_L); 

      printf("image width %d height %d\n",xsize, ysize);

      //Allocate and set the OpenCV calibration structures:
      INTRINSIC_MATRIX_L= cvCreateMat( 3, 3, CV_32FC1);
      DIST_COEFFS_L     = cvCreateMat( 5, 1, CV_32FC1);
      cvmSet(INTRINSIC_MATRIX_L, 0, 0, FOCAL_LENGTHS[0]);
      cvmSet(INTRINSIC_MATRIX_L, 0, 1, 0.0);
      cvmSet(INTRINSIC_MATRIX_L, 0, 2, PRINCIPLE_POINT[0]);
      
      cvmSet(INTRINSIC_MATRIX_L, 1, 0, 0.0);
      cvmSet(INTRINSIC_MATRIX_L, 1, 1, FOCAL_LENGTHS[1]);
      cvmSet(INTRINSIC_MATRIX_L, 1, 2, PRINCIPLE_POINT[1]);
      
      cvmSet(INTRINSIC_MATRIX_L, 2, 0, 0.0);
      cvmSet(INTRINSIC_MATRIX_L, 2, 1, 0.0);
      cvmSet(INTRINSIC_MATRIX_L, 2, 2, 1.0);
      
      DISTORTION[0]= kc[0];
      DISTORTION[1]= kc[1];
      DISTORTION[2]= kc[3];
      DISTORTION[3]= kc[2];
      DISTORTION[4]= kc[4];

      cvmSet(DIST_COEFFS_L, 0, 0, DISTORTION[0]);
      cvmSet(DIST_COEFFS_L, 1, 0, DISTORTION[1]);
      cvmSet(DIST_COEFFS_L, 2, 0, DISTORTION[2]);
      cvmSet(DIST_COEFFS_L, 3, 0, DISTORTION[3]);
      cvmSet(DIST_COEFFS_L, 4, 0, DISTORTION[4]);

      //Display calibration parameters (for debug):
      printf("intrinsic matrix:  %f %f %f\n",cvmGet(INTRINSIC_MATRIX_L,0,0),cvmGet(INTRINSIC_MATRIX_L,0,1),cvmGet(INTRINSIC_MATRIX_L,0,2));
      printf("                   %f %f %f\n",cvmGet(INTRINSIC_MATRIX_L,1,0),cvmGet(INTRINSIC_MATRIX_L,1,1),cvmGet(INTRINSIC_MATRIX_L,1,2));
      printf("                   %f %f %f\n",cvmGet(INTRINSIC_MATRIX_L,2,0),cvmGet(INTRINSIC_MATRIX_L,2,1),cvmGet(INTRINSIC_MATRIX_L,2,2));
      printf("distortion coeffs: %f %f %f %f\n",cvmGet(DIST_COEFFS_L,0,0),cvmGet(DIST_COEFFS_L,1,0),cvmGet(DIST_COEFFS_L,2,0),cvmGet(DIST_COEFFS_L,3,0)); 
      

  
      PARAM_FILE_PATH.clear();
      PARAM_FILE_PATH += initStruct->filePath;
       
      printf("file path %s\n", PARAM_FILE_PATH.c_str());
       
      fileName.clear();
      fileName +=  PARAM_FILE_PATH;
      fileName += "/cal/";
      fileName += "artk_cal_Left.cal";

      fileCal.open(fileName.c_str(), std::ios::trunc);
       
      fileCal << "ARToolKitPlus_CamCal_Rev02 " << std::endl;
  
      fileCal << IMAGE_WIDTH << ' ' <<  IMAGE_HEIGHT << ' ';
      fileCal << cvmGet(INTRINSIC_MATRIX_L,0,2) << ' ' << cvmGet(INTRINSIC_MATRIX_L,1,2) << ' ';
      fileCal << cvmGet(INTRINSIC_MATRIX_L,0,0) << ' ' << cvmGet(INTRINSIC_MATRIX_L,1,1) << ' ';
      fileCal << cvmGet(DIST_COEFFS_L,0,0) << ' ' <<  cvmGet(DIST_COEFFS_L,1,0) << ' ' << cvmGet(DIST_COEFFS_L,2,0) << ' ';
      fileCal << cvmGet(DIST_COEFFS_L,3,0) << ' ' << "0.0 0.0 10" << std::endl;
      fileCal.close();

      cvShowImage("VIMAN_Left", IMAGE_SRC_L);
      cvWaitKey(3);
      break;
      
    case VIMAN_INPUT_VIAM:

      if(SDI_F->flagDisplay==VIMAN_ENABLE) {
	cvNamedWindow((char*)"VIMAN_Left", CV_WINDOW_NORMAL);
	cvResizeWindow("VIMAN_Left", 512, 384);
	if( initStruct->stereo == VIMAN_ENABLE) {
	  cvNamedWindow("VIMAN_Right", CV_WINDOW_NORMAL);
	  cvResizeWindow((char*)"VIMAN_Right", 512, 384);
	}
      }
    
      fprintf(stderr, "looking for poster %s\n", initStruct->viamPosterName);
      if(posterFind(initStruct->viamPosterName, &VIAM_CAMERA_POSTER_ID) != OK) {
	h2perror("vimanInitMain: viman posterFind");
	*report = S_viman_POSTER_NOT_FOUND;
	return ETHER;
      }
       
      if(posterTake(VIAM_CAMERA_POSTER_ID, POSTER_READ) != OK) {
	h2perror("vimanInitMain: viman posterTake");
	*report = S_viman_CANNOT_READ_POSTER;
	return ETHER;
      }

      VIAM_IMAGE = (ViamImageBank *)posterAddr(VIAM_CAMERA_POSTER_ID);
      if(VIAM_IMAGE == NULL) {
	posterGive(VIAM_CAMERA_POSTER_ID);
	*report = S_viman_CANNOT_READ_POSTER;
	return ETHER;
      }

      if(VIAM_IMAGE->nImages <= 0) {
	printf("vimanInitMain : There is no image acquired!\n");
	*report = S_viman_CANNOT_READ_POSTER;
	return ETHER;        
      }



      if( initStruct->stereo == VIMAN_DISABLE) {
	VIAM_CAMERA_LEFT_ID = 0;
      } else {
	if(strcmp(initStruct->viamImageLeftName, VIAM_IMAGE->image[0].name.id)==0) {
	  VIAM_CAMERA_LEFT_ID = 0;
	  VIAM_CAMERA_RIGHT_ID = 1;
	}else {
	  VIAM_CAMERA_LEFT_ID = 1;
	  VIAM_CAMERA_RIGHT_ID = 0;
	}
      }


      VIAM_RATIO_BASELINE = initStruct->stereoRatioBaseline;
      SDI_F->corImageFrame.dx = 0.0;
      SDI_F->corImageFrame.dy = 0.0;
      SDI_F->corImageFrame.dz = 0.0;

      SDI_F->corOriginFrame.dx = 0.0;
      SDI_F->corOriginFrame.dy = 0.0;
      SDI_F->corOriginFrame.dz = 0.0;

	//initStruct->stereoDephtCorrection;

      printf("viman set CameraLeftID to %d\n",  VIAM_CAMERA_LEFT_ID);
      if( initStruct->stereo == VIMAN_ENABLE) {
	printf("viman set CameraRightID to %d\n",  VIAM_CAMERA_RIGHT_ID);
      }
      printf("viman set BASELINE RATIO to %f\n",  VIAM_RATIO_BASELINE);
      printf("viman set SDI_F->corImageFrame.dz to %f\n", SDI_F->corImageFrame.dz);

      IMAGE_WIDTH = VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].width;
      IMAGE_HEIGHT= VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].height;
      NB_PIXELS = (size_t)(IMAGE_WIDTH*IMAGE_HEIGHT);
      printf("viman set Left WIDTH %d HEIGHT  %d\n", IMAGE_WIDTH,IMAGE_HEIGHT);
      //left
      if(initStruct->color == VIMAN_ENABLE) {
	IMAGE_SRC_L   = cvCreateImage(cvSize(IMAGE_WIDTH, IMAGE_HEIGHT), 8, 3);
	memcpy(IMAGE_SRC_L->imageData, VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].data+VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].dataOffset,VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].imageSize);
	CAMERA_BUFFER_L = new unsigned char[NB_PIXELS];
	IMAGE_GRAY_L  = cvCreateImage(cvSize(IMAGE_WIDTH, IMAGE_HEIGHT), 8, 1);
	IMAGE_GRAY_EQUALIZED_L  = cvCreateImage(cvSize(IMAGE_WIDTH, IMAGE_HEIGHT), 8, 1);
	IMAGE_UNDIST_L= cvCreateImage(cvSize(IMAGE_WIDTH, IMAGE_HEIGHT), 8, 1);
	IMAGE_MOD_L   = cvCreateImage(cvSize(IMAGE_WIDTH, IMAGE_HEIGHT), 8, 3);
	cvCvtColor(IMAGE_SRC_L, IMAGE_GRAY_L, CV_BGR2GRAY);
	cvEqualizeHist(IMAGE_GRAY_L, IMAGE_GRAY_EQUALIZED_L);
	cvCopy(IMAGE_GRAY_EQUALIZED_L, IMAGE_UNDIST_L, 0);
      } else {
	IMAGE_SRC_L   = cvCreateImage(cvSize(IMAGE_WIDTH, IMAGE_HEIGHT), 8, 1);
	memcpy(IMAGE_SRC_L->imageData, VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].data+VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].dataOffset,VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].imageSize);
	CAMERA_BUFFER_L = new unsigned char[NB_PIXELS];
	IMAGE_GRAY_L  = cvCreateImage(cvSize(IMAGE_WIDTH, IMAGE_HEIGHT), 8, 1);
	IMAGE_GRAY_EQUALIZED_L  = cvCreateImage(cvSize(IMAGE_WIDTH, IMAGE_HEIGHT), 8, 1);
	IMAGE_UNDIST_L= cvCreateImage(cvSize(IMAGE_WIDTH, IMAGE_HEIGHT), 8, 1);
	IMAGE_MOD_L   = cvCreateImage(cvSize(IMAGE_WIDTH, IMAGE_HEIGHT), 8, 3);
	cvCopy(IMAGE_SRC_L, IMAGE_GRAY_L, 0);
	cvEqualizeHist(IMAGE_GRAY_L, IMAGE_GRAY_EQUALIZED_L);
	cvCopy(IMAGE_GRAY_EQUALIZED_L, IMAGE_UNDIST_L, 0);
      }
 
      cvMerge( IMAGE_UNDIST_L, IMAGE_UNDIST_L, IMAGE_UNDIST_L, NULL, IMAGE_MOD_L); 

      //Allocate and set the OpenCV calibration structures:
      INTRINSIC_MATRIX_L= cvCreateMat( 3, 3, CV_32FC1);
      DIST_COEFFS_L     = cvCreateMat( 5, 1, CV_32FC1);  

      if( VIAM_IMAGE->calibration.type == VIAM_CAL_STEREO) {
	cvmSet(INTRINSIC_MATRIX_L, 0, 0, VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].calibration.intrirect[0]);
	cvmSet(INTRINSIC_MATRIX_L, 0, 1, 0.0);
	cvmSet(INTRINSIC_MATRIX_L, 0, 2, VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].calibration.intrirect[2]);
	
	cvmSet(INTRINSIC_MATRIX_L, 1, 0, 0.0);
	cvmSet(INTRINSIC_MATRIX_L, 1, 1, VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].calibration.intrirect[4]);
	cvmSet(INTRINSIC_MATRIX_L, 1, 2, VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].calibration.intrirect[5]);
      
      } else {
	cvmSet(INTRINSIC_MATRIX_L, 0, 0, VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].calibration.intrinsic[0]);
	cvmSet(INTRINSIC_MATRIX_L, 0, 1, 0.0);
	cvmSet(INTRINSIC_MATRIX_L, 0, 2, VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].calibration.intrinsic[2]);
	
	cvmSet(INTRINSIC_MATRIX_L, 1, 0, 0.0);
	cvmSet(INTRINSIC_MATRIX_L, 1, 1, VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].calibration.intrinsic[4]);
	cvmSet(INTRINSIC_MATRIX_L, 1, 2, VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].calibration.intrinsic[5]);
      }
			    
      cvmSet(INTRINSIC_MATRIX_L, 2, 0, 0.0);
      cvmSet(INTRINSIC_MATRIX_L, 2, 1, 0.0);
      cvmSet(INTRINSIC_MATRIX_L, 2, 2, 1.0);
              
      if(CORRECT_DISTORSION == VIMAN_ENABLE) { 
	cvmSet(DIST_COEFFS_L, 0, 0, VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].calibration.distortion[0]);
	cvmSet(DIST_COEFFS_L, 1, 0, VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].calibration.distortion[1]);
	cvmSet(DIST_COEFFS_L, 2, 0, VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].calibration.distortion[2]);
	cvmSet(DIST_COEFFS_L, 3, 0, VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].calibration.distortion[3]);  
	cvmSet(DIST_COEFFS_L, 4, 0, VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].calibration.distortion[4]);  
      } else {		   
	cvmSet(DIST_COEFFS_L, 0, 0, 0.0);
	cvmSet(DIST_COEFFS_L, 1, 0, 0.0);
	cvmSet(DIST_COEFFS_L, 2, 0, 0.0);
	cvmSet(DIST_COEFFS_L, 3, 0, 0.0);  
	cvmSet(DIST_COEFFS_L, 4, 0, 0.0);  
      }
      for(int u=0; u<9; u++) {
	ROTATION_MATRIX_L[u] =  VIAM_IMAGE->image[VIAM_CAMERA_LEFT_ID].calibration.rotation[u];
      }
      PARAM_FILE_PATH.clear();
      PARAM_FILE_PATH += initStruct->filePath;
      printf("file path %s\n", PARAM_FILE_PATH.c_str());
      fileName.clear();
      fileName +=  PARAM_FILE_PATH;
      fileName += "/cal/";
      fileName += "artk_cal_Left.cal";
      fileCal.open(fileName.c_str(), std::ios::trunc);
   
      fileCal << "ARToolKitPlus_CamCal_Rev02 " << std::endl;
      fileCal << IMAGE_WIDTH << ' ' <<  IMAGE_HEIGHT << ' ';
      fileCal << cvmGet(INTRINSIC_MATRIX_L,0,2) << ' ' << cvmGet(INTRINSIC_MATRIX_L,1,2) << ' ';
      fileCal << cvmGet(INTRINSIC_MATRIX_L,0,0) << ' ' << cvmGet(INTRINSIC_MATRIX_L,1,1) << ' ';
      fileCal << cvmGet(DIST_COEFFS_L,0,0) << ' ' <<  cvmGet(DIST_COEFFS_L,1,0) << ' ' << cvmGet(DIST_COEFFS_L,2,0) << ' ';
      fileCal << cvmGet(DIST_COEFFS_L,3,0) << ' ' << "0.0 0.0 10" << std::endl;
      fileCal.close();
      //Display calibration parameters (for debug):
      printf("image Left\n:"); 
      printf("intrinsic matrix:  %f %f %f\n",cvmGet(INTRINSIC_MATRIX_L,0,0),cvmGet(INTRINSIC_MATRIX_L,0,1),cvmGet(INTRINSIC_MATRIX_L,0,2));
      printf("                   %f %f %f\n",cvmGet(INTRINSIC_MATRIX_L,1,0),cvmGet(INTRINSIC_MATRIX_L,1,1),cvmGet(INTRINSIC_MATRIX_L,1,2));
      printf("                   %f %f %f\n",cvmGet(INTRINSIC_MATRIX_L,2,0),cvmGet(INTRINSIC_MATRIX_L,2,1),cvmGet(INTRINSIC_MATRIX_L,2,2));
      printf("distortion coeffs: %f %f %f %f\n",cvmGet(DIST_COEFFS_L,0,0),cvmGet(DIST_COEFFS_L,1,0),cvmGet(DIST_COEFFS_L,2,0),cvmGet(DIST_COEFFS_L,3,0));
      cvShowImage("VIMAN_Left", IMAGE_SRC_L);


      if( initStruct->stereo == VIMAN_ENABLE) {
	//right
	if(initStruct->color == VIMAN_ENABLE) {
	  printf("viman create image to be use with color\n");
	  IMAGE_SRC_R   = cvCreateImage(cvSize(IMAGE_WIDTH, IMAGE_HEIGHT), 8, 3);
	  memcpy(IMAGE_SRC_R->imageData, VIAM_IMAGE->image[VIAM_CAMERA_RIGHT_ID].data+VIAM_IMAGE->image[VIAM_CAMERA_RIGHT_ID].dataOffset,VIAM_IMAGE->image[VIAM_CAMERA_RIGHT_ID].imageSize);
	  CAMERA_BUFFER_R = new unsigned char[NB_PIXELS];
	  IMAGE_GRAY_R  = cvCreateImage(cvSize(IMAGE_WIDTH, IMAGE_HEIGHT), 8, 1);
	  IMAGE_GRAY_EQUALIZED_R  = cvCreateImage(cvSize(IMAGE_WIDTH, IMAGE_HEIGHT), 8, 1);
	  IMAGE_UNDIST_R= cvCreateImage(cvSize(IMAGE_WIDTH, IMAGE_HEIGHT), 8, 1);
	  IMAGE_MOD_R   = cvCreateImage(cvSize(IMAGE_WIDTH, IMAGE_HEIGHT), 8, 3);
	  cvCvtColor(IMAGE_SRC_R, IMAGE_GRAY_R, CV_BGR2GRAY);
	  cvEqualizeHist(IMAGE_GRAY_R, IMAGE_GRAY_EQUALIZED_R);
	  cvCopy(IMAGE_GRAY_EQUALIZED_R, IMAGE_UNDIST_R, 0);
	} else {
	  printf("viman create image to be use with B&W\n");
	  IMAGE_SRC_R   = cvCreateImage(cvSize(IMAGE_WIDTH, IMAGE_HEIGHT), 8, 1);
	  memcpy(IMAGE_SRC_R->imageData, VIAM_IMAGE->image[VIAM_CAMERA_RIGHT_ID].data+VIAM_IMAGE->image[VIAM_CAMERA_RIGHT_ID].dataOffset,VIAM_IMAGE->image[VIAM_CAMERA_RIGHT_ID].imageSize);
	  CAMERA_BUFFER_R = new unsigned char[NB_PIXELS];
	  IMAGE_GRAY_R  = cvCreateImage(cvSize(IMAGE_WIDTH, IMAGE_HEIGHT), 8, 1);
	  IMAGE_GRAY_EQUALIZED_R  = cvCreateImage(cvSize(IMAGE_WIDTH, IMAGE_HEIGHT), 8, 1);
	  IMAGE_UNDIST_R= cvCreateImage(cvSize(IMAGE_WIDTH, IMAGE_HEIGHT), 8, 1);
	  IMAGE_MOD_R   = cvCreateImage(cvSize(IMAGE_WIDTH, IMAGE_HEIGHT), 8, 3);
	  cvCopy(IMAGE_SRC_R, IMAGE_GRAY_R, 0);
	  cvEqualizeHist(IMAGE_GRAY_R, IMAGE_GRAY_EQUALIZED_R);
	  cvCopy(IMAGE_GRAY_EQUALIZED_R, IMAGE_UNDIST_R, 0);
	}

	cvMerge( IMAGE_UNDIST_R, IMAGE_UNDIST_R, IMAGE_UNDIST_R, NULL, IMAGE_MOD_R); 
	//Allocate and set the OpenCV calibration structures:
	INTRINSIC_MATRIX_R= cvCreateMat( 3, 3, CV_32FC1);
	DIST_COEFFS_R     = cvCreateMat( 5, 1, CV_32FC1);  
	cvmSet(INTRINSIC_MATRIX_R, 0, 0, VIAM_IMAGE->image[VIAM_CAMERA_RIGHT_ID].calibration.intrirect[0]);
	cvmSet(INTRINSIC_MATRIX_R, 0, 1, 0.0);
	cvmSet(INTRINSIC_MATRIX_R, 0, 2, VIAM_IMAGE->image[VIAM_CAMERA_RIGHT_ID].calibration.intrirect[2]);
       			   
	cvmSet(INTRINSIC_MATRIX_R, 1, 0, 0.0);
	cvmSet(INTRINSIC_MATRIX_R, 1, 1, VIAM_IMAGE->image[VIAM_CAMERA_RIGHT_ID].calibration.intrirect[4]);
	cvmSet(INTRINSIC_MATRIX_R, 1, 2, VIAM_IMAGE->image[VIAM_CAMERA_RIGHT_ID].calibration.intrirect[5]);
			   
	cvmSet(INTRINSIC_MATRIX_R, 2, 0, 0.0);
	cvmSet(INTRINSIC_MATRIX_R, 2, 1, 0.0);
	cvmSet(INTRINSIC_MATRIX_R, 2, 2, 1.0);
              
	if(CORRECT_DISTORSION == VIMAN_ENABLE) { 
	  cvmSet(DIST_COEFFS_R, 0, 0, VIAM_IMAGE->image[VIAM_CAMERA_RIGHT_ID].calibration.distortion[0]);
	  cvmSet(DIST_COEFFS_R, 1, 0, VIAM_IMAGE->image[VIAM_CAMERA_RIGHT_ID].calibration.distortion[1]);
	  cvmSet(DIST_COEFFS_R, 2, 0, VIAM_IMAGE->image[VIAM_CAMERA_RIGHT_ID].calibration.distortion[2]);
	  cvmSet(DIST_COEFFS_R, 3, 0, VIAM_IMAGE->image[VIAM_CAMERA_RIGHT_ID].calibration.distortion[3]); 
	  cvmSet(DIST_COEFFS_R, 4, 0, VIAM_IMAGE->image[VIAM_CAMERA_RIGHT_ID].calibration.distortion[4]); 
	} else {		   
	  cvmSet(DIST_COEFFS_R, 0, 0, 0.0);
	  cvmSet(DIST_COEFFS_R, 1, 0, 0.0);
	  cvmSet(DIST_COEFFS_R, 2, 0, 0.0);
	  cvmSet(DIST_COEFFS_R, 3, 0, 0.0); 
	  cvmSet(DIST_COEFFS_R, 4, 0, 0.0); 
	}
	for(int u=0; u<9; u++) {
	  ROTATION_MATRIX_R[u] =  VIAM_IMAGE->image[VIAM_CAMERA_RIGHT_ID].calibration.rotation[u];
	}
	PARAM_FILE_PATH.clear();
	PARAM_FILE_PATH += initStruct->filePath;       
	printf("file path %s\n", PARAM_FILE_PATH.c_str());
	fileName.clear();
	fileName +=  PARAM_FILE_PATH;
	fileName += "/cal/";
	fileName += "artk_cal_Right.cal";
	fileCal.open(fileName.c_str(), std::ios::trunc);
       
	fileCal << "ARToolKitPlus_CamCal_Rev02 " << std::endl;
	fileCal << IMAGE_WIDTH << ' ' <<  IMAGE_HEIGHT << ' ';
	fileCal << cvmGet(INTRINSIC_MATRIX_R,0,2) << ' ' << cvmGet(INTRINSIC_MATRIX_R,1,2) << ' ';
	fileCal << cvmGet(INTRINSIC_MATRIX_R,0,0) << ' ' << cvmGet(INTRINSIC_MATRIX_R,1,1) << ' ';
	fileCal << cvmGet(DIST_COEFFS_R,0,0) << ' ' <<  cvmGet(DIST_COEFFS_R,1,0) << ' ' << cvmGet(DIST_COEFFS_R,2,0) << ' ';
	fileCal << cvmGet(DIST_COEFFS_R,3,0) << ' ' << "0.0 0.0 10" << std::endl;
	fileCal.close();
	printf("image Right\n:"); 
	printf("intrinsic matrix:  %f %f %f\n",cvmGet(INTRINSIC_MATRIX_R,0,0),cvmGet(INTRINSIC_MATRIX_R,0,1),cvmGet(INTRINSIC_MATRIX_R,0,2));
	printf("                   %f %f %f\n",cvmGet(INTRINSIC_MATRIX_R,1,0),cvmGet(INTRINSIC_MATRIX_R,1,1),cvmGet(INTRINSIC_MATRIX_R,1,2));
	printf("                   %f %f %f\n",cvmGet(INTRINSIC_MATRIX_R,2,0),cvmGet(INTRINSIC_MATRIX_R,2,1),cvmGet(INTRINSIC_MATRIX_R,2,2));
	printf("distortion coeffs: %f %f %f %f\n",cvmGet(DIST_COEFFS_R,0,0),cvmGet(DIST_COEFFS_R,1,0),cvmGet(DIST_COEFFS_R,2,0),cvmGet(DIST_COEFFS_R,3,0));     
	cvShowImage("VIMAN_Right", IMAGE_SRC_R);
      }

      posterGive(VIAM_CAMERA_POSTER_ID);   
      cvWaitKey(3);
      break;
    }
  
  //Read the object description file:
  fileName.clear();
  fileName+= PARAM_FILE_PATH;
  fileName+= "/objectList_cfg";
  if( !vimanReadObjectListFile(fileName) ) {
    *report= S_viman_CANNOT_READ_CONFIG_FILE;
    return ETHER;
  }
  
  //For each object, read its .cfg file:
  for(i=0; i<SDI_F->objectArray.nbObjects; i++) {
    objectName= SDI_F->objectArray.objects[i].name;
    fileName= SDI_F->objectArray.objects[i].file;
    SDI_F->objectArray.objects[i].track= VIMAN_ENABLE;
    SDI_F->objectArray.objects[i].calibrate= VIMAN_DISABLE;
    SDI_F->objectArray.objects[i].marker_visibility_L= 0;
    SDI_F->objectArray.objects[i].marker_visibility_R= 0;
    if( !vimanReadObjectCfgFile(objectName, fileName) ) {
      *report= S_viman_CANNOT_READ_CONFIG_FILE;
      return ETHER;
    }
  }
  
  //Display the content of objectArray (for debug only):
  /*
    for(i=0; i<SDI_F->objectArray.nbObjects; i++)
    {
    printf("object %s: \n", SDI_F->objectArray.objects[i].name);
    printf("\t nbMarkers: %d\n", SDI_F->objectArray.objects[i].nbMarkers);
    for(int j=0; j<SDI_F->objectArray.objects[i].nbMarkers; j++)
    {
    printf("\t\t ID: %d\n", SDI_F->objectArray.objects[i].markerIDs[j]);
    printf("\t\t pattern width: %f\n", SDI_F->objectArray.objects[i].patternWidths[j]);
    printf("\t\t pose: %f %f %f %f\n", SDI_F->objectArray.objects[i].relativePoses[j].nx, SDI_F->objectArray.objects[i].relativePoses[j].ox, SDI_F->objectArray.objects[i].relativePoses[j].ax, SDI_F->objectArray.objects[i].relativePoses[j].px);
    printf("\t\t       %f %f %f %f\n", SDI_F->objectArray.objects[i].relativePoses[j].ny, SDI_F->objectArray.objects[i].relativePoses[j].oy, SDI_F->objectArray.objects[i].relativePoses[j].ay, SDI_F->objectArray.objects[i].relativePoses[j].py);
    printf("\t\t       %f %f %f %f\n", SDI_F->objectArray.objects[i].relativePoses[j].nz, SDI_F->objectArray.objects[i].relativePoses[j].oz, SDI_F->objectArray.objects[i].relativePoses[j].az, SDI_F->objectArray.objects[i].relativePoses[j].pz);
    }
    }
  */

  for(int u=0; u<VIMAN_MAX_OBJ_NB; u++) {
    ROTATION_VECTOR_L[u]   = cvCreateMat( 3, 1, CV_32FC1 );
    ROTATION_VECTOR_MARKER_L[u]   = cvCreateMat( 3, 1, CV_32FC1 );
    TRANSLATION_VECTOR_L[u]= cvCreateMat( 3, 1, CV_32FC1 );
    ROTATION_VECTOR_R[u]   = cvCreateMat( 3, 1, CV_32FC1 );
    TRANSLATION_VECTOR_R[u]= cvCreateMat( 3, 1, CV_32FC1 );
  }

  SDI_F->vimanInitIsDone= VIMAN_ENABLE;


  /* Initialize the structures for the imagesOut */

  IMAGE_OUT_L  = cvCreateImage(cvSize(IMAGE_OUT_WIDTH, IMAGE_OUT_HEIGHT), 8, 3);
  IMAGE_OUT_R  = cvCreateImage(cvSize(IMAGE_OUT_WIDTH, IMAGE_OUT_HEIGHT), 8, 3);

  // Size of a complet ViamImageBank Structure bankHeader + nbImage*imageHeader + nbColor*nbImages*resolution
  VIMAN_BANK_SIZE= sizeof(ViamImageBank) + VIAM_IMAGE->nImages*sizeof(ViamImageHeader) 
   + VIAM_IMAGE->nImages*IMAGE_OUT_L->imageSize;

  // Memory allocation
  VIMAN_BANK = (ViamImageBank*)malloc(VIMAN_BANK_SIZE);

  /********  Create poster_  **********/
  if (posterCreate(VIMAN_IMAGE_OUT_POSTER_NAME, VIMAN_BANK_SIZE, &(VIMAN_BANK_POSTER_ID)) != OK) {
    printf("ERROR: could not create poster %s\n", VIMAN_IMAGE_OUT_POSTER_NAME);
    return ETHER;
  } else {
    printf("viman: poster %s succesfully created\n", VIMAN_IMAGE_OUT_POSTER_NAME);
  }
  /**********************************/

  /* Fill the image information */

  cvResize(IMAGE_MOD_L, IMAGE_OUT_L);
  cvResize(IMAGE_MOD_R, IMAGE_OUT_R);
 
  strcpy(VIMAN_BANK->name.id,"Images");
  // strcpy(VIMAN_BANK->name.id, VIAM_IMAGE->name.id);
  VIMAN_BANK->calibration.type= VIAM_IMAGE->calibration.type;
  VIMAN_BANK->calibration.baseline= VIAM_IMAGE->calibration.baseline;
  VIMAN_BANK->calibration.pbaseline=VIAM_IMAGE->calibration.pbaseline;
  VIMAN_BANK->nImages = VIAM_IMAGE->nImages;

  VIMAN_BANK->image[0].calibration =  VIAM_IMAGE->image[0].calibration;
  strcpy(VIMAN_BANK->image[0].name.id, VIAM_IMAGE->image[0].name.id); 
  VIMAN_BANK->image[0].tacq_sec   =	    VIAM_IMAGE->image[0].tacq_sec;
  VIMAN_BANK->image[0].tacq_usec  =      VIAM_IMAGE->image[0].tacq_usec;
  VIMAN_BANK->image[0].nChannels  =      IMAGE_OUT_L->nChannels;
  VIMAN_BANK->image[0].depth      =      IMAGE_OUT_L->depth;
  VIMAN_BANK->image[0].width      =      IMAGE_OUT_L->width;
  VIMAN_BANK->image[0].height     =      IMAGE_OUT_L->height;
  VIMAN_BANK->image[0].imageSize  =      IMAGE_OUT_L->imageSize;
  VIMAN_BANK->image[0].widthStep  =      IMAGE_OUT_L->widthStep; 
  VIMAN_BANK->image[0].dataOffset =      sizeof(ViamImageHeader); // memory: header[0]/header[1]/data[0]/data[1]

  memcpy(VIMAN_BANK->image[0].data + VIMAN_BANK->image[0].dataOffset, 
	 IMAGE_OUT_L->imageData, IMAGE_OUT_L->imageSize);


  if( VIAM_IMAGE->nImages > 1) {
    VIMAN_BANK->image[1].calibration =  VIAM_IMAGE->image[1].calibration;
    strcpy(VIMAN_BANK->image[1].name.id, VIAM_IMAGE->image[1].name.id); 
    VIMAN_BANK->image[1].tacq_sec   =	    VIAM_IMAGE->image[1].tacq_sec;
    VIMAN_BANK->image[1].tacq_usec  =      VIAM_IMAGE->image[1].tacq_usec;
    VIMAN_BANK->image[1].nChannels  =      IMAGE_OUT_R->nChannels;
    VIMAN_BANK->image[1].depth      =      IMAGE_OUT_R->depth;
    VIMAN_BANK->image[1].width      =      IMAGE_OUT_R->width;
    VIMAN_BANK->image[1].height     =      IMAGE_OUT_R->height;
    VIMAN_BANK->image[1].imageSize  =      IMAGE_OUT_R->imageSize;
    VIMAN_BANK->image[1].widthStep  =      IMAGE_OUT_R->widthStep; /* width must be */ 
    VIMAN_BANK->image[1].dataOffset =      IMAGE_OUT_L->imageSize; 

    memcpy(VIMAN_BANK->image[1].data + VIMAN_BANK->image[1].dataOffset, 
	   IMAGE_OUT_R->imageData, IMAGE_OUT_R->imageSize);
  }

 
  //writing to poster_
  if( posterWrite(VIMAN_BANK_POSTER_ID, 0, VIMAN_BANK, VIMAN_BANK_SIZE) == ERROR )
    h2perror("Write error");

  printf("***************************************************************************\n");
  printf("INFO: You can view the images using picoweb, an url example is:\n");
  printf("    http://${REMOTE_HOST}:8080/viman?bank=Images&image=CameraTopLeft&quality=65&delay=10 \n");
  printf("***************************************************************************\n");
  return ETHER;
}

ACTIVITY_EVENT
vimanActivateTrackingMain(int *report)
{   
  const char* description;
  MyLogger logger;
  std::string filename;

  if(TRACKER_L!=NULL) {
    delete TRACKER_L;
    TRACKER_L= NULL;
  }
  
  //  - 6x6 sized marker images
  //  - samples at a maximum of 6x6
  //  - works with luminance (gray) images
  //  - can load a maximum of 1 pattern
  //  - can detect a maximum of 16 patterns in one image
  TRACKER_L = new ARToolKitPlus::TrackerSingleMarkerImpl<6,6,6, 1, 16>(IMAGE_WIDTH, IMAGE_HEIGHT);

  TRACKER_L->setPixelFormat(ARToolKitPlus::PIXEL_FORMAT_LUM);

  description = TRACKER_L->getDescription();

  if(PRINT_INFO) 
    printf("ARToolKitPlus compile-time information:\n%s\n\n", description);

  if(PRINT_INFO) 
    TRACKER_L->setLogger(&logger);


  // the marker in the BCH test image has a thiner border...
  TRACKER_L->setBorderWidth(0.25);

  //TRACKER_L->setPatternWidth(PATTERN_WIDTH);

  // set a threshold. we could also activate automatic thresholding
  TRACKER_L->activateAutoThreshold(true);
  TRACKER_L->setNumAutoThresholdRetries(10);
  //TRACKER_L->setThreshold(150);
  // let's use lookup-table undistortion for high-speed
  // note: LUT only works with images up to 1024x1024
  TRACKER_L->setUndistortionMode(ARToolKitPlus::UNDIST_NONE);

  TRACKER_L->setImageProcessingMode(ARToolKitPlus::IMAGE_FULL_RES);

  // RPP is more robust than ARToolKit's standard pose estimator
  //TRACKER_L->setPoseEstimator(ARToolKitPlus::POSE_ESTIMATOR_RPP);

  //TRACKER_L->setMarkerMode(ARToolKitPlus::MARKER_ID_BCH);
  TRACKER_L->setMarkerMode(ARToolKitPlus::MARKER_ID_SIMPLE);

  
  filename.clear();
  filename+= PARAM_FILE_PATH;
  filename+= "/cal/";
  filename+= "artk_cal_Left.cal";
  
  if(!TRACKER_L->init(filename.c_str(), 1.0f, 1000.0f)) {
    printf("ERROR: init() failed\n");
    delete TRACKER_L;
    TRACKER_L= NULL;
    *report= S_viman_TRACKING_ACTIVATION_FAILED;
    return ETHER;
  }

  if(TRACKER_R!=NULL) {
    delete TRACKER_R;
    TRACKER_R= NULL;
  }

  printf("tracker left OK\n");
  if(SDI_F->initStruct.stereo == VIMAN_DISABLE) {

    printf("vimanActivateTrackingMain tracker initialized\n");
    SDI_F->flagTracking= VIMAN_ENABLE;

    return ETHER;

  }

  if(TRACKER_R!=NULL) {
    delete TRACKER_R;
    TRACKER_R= NULL;
  }

  //  - 6x6 sized marker images
  //  - samples at a maximum of 6x6
  //  - works with luminance (gray) images
  //  - can load a maximum of 1 pattern
  //  - can detect a maximum of 16 patterns in one image
  TRACKER_R = new ARToolKitPlus::TrackerSingleMarkerImpl<6,6,6, 1, 16>(IMAGE_WIDTH, IMAGE_HEIGHT);

  TRACKER_R->setPixelFormat(ARToolKitPlus::PIXEL_FORMAT_LUM);

  description = TRACKER_R->getDescription();

  if(PRINT_INFO) 
     printf("ARToolKitPlus compile-time information:\n%s\n\n", description);

  if(PRINT_INFO) 
     TRACKER_R->setLogger(&logger);


  // the marker in the BCH test image has a thiner border...
  TRACKER_R->setBorderWidth(0.25);

  //TRACKER_R->setPatternWidth(PATTERN_WIDTH);

  // set a threshold. we could also activate automatic thresholding
  TRACKER_R->activateAutoThreshold(true);
  TRACKER_R->setNumAutoThresholdRetries(10);
  //TRACKER_R->setThreshold(150);
  // let's use lookup-table undistortion for high-speed
  // note: LUT only works with images up to 1024x1024
  TRACKER_R->setUndistortionMode(ARToolKitPlus::UNDIST_NONE);

  TRACKER_R->setImageProcessingMode(ARToolKitPlus::IMAGE_FULL_RES);

  // RPP is more robust than ARToolKit's standard pose estimator
  //TRACKER_R->setPoseEstimator(ARToolKitPlus::POSE_ESTIMATOR_RPP);

 //TRACKER_R->setMarkerMode(ARToolKitPlus::MARKER_ID_BCH);
  TRACKER_R->setMarkerMode(ARToolKitPlus::MARKER_ID_SIMPLE);

  
  filename.clear();
  filename+= PARAM_FILE_PATH;
  filename+= "/cal/";
  filename+= "artk_cal_Right.cal";
  
  if(!TRACKER_R->init(filename.c_str(), 1.0f, 1000.0f)) {
    printf("ERROR: init() failed\n");
    delete TRACKER_R;
    TRACKER_R= NULL;
    *report= S_viman_TRACKING_ACTIVATION_FAILED;
    return ETHER;
  }

  printf("vimanActivateTrackingMain tracker initialized\n");
  SDI_F->flagTracking= VIMAN_ENABLE;

  return ETHER;
}

ACTIVITY_EVENT
vimanSetThresholdARToolKitMain(int *thresholdARToolKit, int *report)
{
  return ETHER;
}

// Draw a the image of a box in the specified image.
// The function receives the camera parameters and the pose of the box in space (rotation and translation vector).
// The edge length of the box is 'size'.
void vimanDrawBox(IplImage* image, const CvMat* intrinsic_matrix, const CvMat* distortion_coeffs, CvMat* rotation_vector, CvMat* translation_vector, double size)
{
  int i;
  CvScalar color;
  CvPoint p[8];
  CvMat *box_points  = cvCreateMat( 8, 3, CV_32FC1);
  CvMat *image_points= cvCreateMat( 8, 2, CV_32FC1);

  cvmSet(box_points, 0, 0, -0.5*size); 
  cvmSet(box_points, 0, 1, -0.5*size); 
  cvmSet(box_points, 0, 2,  0.0); 

  cvmSet(box_points, 1, 0,  0.5*size); 
  cvmSet(box_points, 1, 1, -0.5*size); 
  cvmSet(box_points, 1, 2,  0.0); 

  cvmSet(box_points, 2, 0,  0.5*size); 
  cvmSet(box_points, 2, 1,  0.5*size); 
  cvmSet(box_points, 2, 2,  0.0); 

  cvmSet(box_points, 3, 0, -0.5*size); 
  cvmSet(box_points, 3, 1,  0.5*size); 
  cvmSet(box_points, 3, 2,  0.0); 

  cvmSet(box_points, 4, 0, -0.5*size); 
  cvmSet(box_points, 4, 1, -0.5*size); 
  cvmSet(box_points, 4, 2,  0.5*size); 

  cvmSet(box_points, 5, 0,  0.5*size); 
  cvmSet(box_points, 5, 1, -0.5*size); 
  cvmSet(box_points, 5, 2,  0.5*size); 

  cvmSet(box_points, 6, 0,  0.5*size); 
  cvmSet(box_points, 6, 1,  0.5*size); 
  cvmSet(box_points, 6, 2,  0.5*size); 

  cvmSet(box_points, 7, 0, -0.5*size); 
  cvmSet(box_points, 7, 1,  0.5*size); 
  cvmSet(box_points, 7, 2,  0.5*size);

  cvProjectPoints2(box_points, rotation_vector, translation_vector, intrinsic_matrix, distortion_coeffs, image_points);

  for(i=0; i<8; i++)
  {
    p[i].x= (int) cvmGet(image_points, i, 0);
    p[i].y= (int) cvmGet(image_points, i, 1); 
  }

  //draw the box edges:
  color= CV_RGB(0, 255, 255);
  cvLine(image, p[0] , p[1], color,  2, CV_AA, 0);
  cvLine(image, p[1] , p[2], color,  2, CV_AA, 0);
  cvLine(image, p[2] , p[3], color,  2, CV_AA, 0);
  cvLine(image, p[3] , p[0], color,  2, CV_AA, 0);

  cvLine(image, p[4] , p[5], color,  2, CV_AA, 0);
  cvLine(image, p[5] , p[6], color,  2, CV_AA, 0);
  cvLine(image, p[6] , p[7], color,  2, CV_AA, 0);
  cvLine(image, p[7] , p[4], color,  2, CV_AA, 0);

  cvLine(image, p[0] , p[4], color,  2, CV_AA, 0);
  cvLine(image, p[1] , p[5], color,  2, CV_AA, 0);
  cvLine(image, p[2] , p[6], color,  2, CV_AA, 0);
  cvLine(image, p[3] , p[7], color,  2, CV_AA, 0);

  //draw the box points as small circles:
  color= CV_RGB(255, 0, 255);
  for(i=0; i<8; i++)
  {
    cvCircle(image, p[i],  3, color,  0,  CV_AA,  0); 
  }

  cvReleaseMat(&box_points);
  cvReleaseMat(&image_points);
}


// The same function as vimanDrawBox except that it receives the box dimensions in along the three axes.
void vimanDrawBox2(IplImage* image, const CvMat* intrinsic_matrix, const CvMat* distortion_coeffs, CvMat* rotation_vector, CvMat* translation_vector, double dX, double dY, double dZ)
{
  int i;
  CvScalar color;
  CvPoint p[8];
  CvMat *box_points  = cvCreateMat( 8, 3, CV_32FC1);
  CvMat *image_points= cvCreateMat( 8, 2, CV_32FC1);

  cvmSet(box_points, 0, 0, -0.5*dX); 
  cvmSet(box_points, 0, 1, -0.5*dY); 
  cvmSet(box_points, 0, 2,  0.0); 

  cvmSet(box_points, 1, 0,  0.5*dX); 
  cvmSet(box_points, 1, 1, -0.5*dY); 
  cvmSet(box_points, 1, 2,  0.0); 

  cvmSet(box_points, 2, 0,  0.5*dX); 
  cvmSet(box_points, 2, 1,  0.5*dY); 
  cvmSet(box_points, 2, 2,  0.0); 

  cvmSet(box_points, 3, 0, -0.5*dX); 
  cvmSet(box_points, 3, 1,  0.5*dY); 
  cvmSet(box_points, 3, 2,  0.0); 

  cvmSet(box_points, 4, 0, -0.5*dX); 
  cvmSet(box_points, 4, 1, -0.5*dY); 
  cvmSet(box_points, 4, 2,  dZ); 

  cvmSet(box_points, 5, 0,  0.5*dX); 
  cvmSet(box_points, 5, 1, -0.5*dY); 
  cvmSet(box_points, 5, 2,  dZ); 

  cvmSet(box_points, 6, 0,  0.5*dX); 
  cvmSet(box_points, 6, 1,  0.5*dY); 
  cvmSet(box_points, 6, 2,  dZ); 

  cvmSet(box_points, 7, 0, -0.5*dX); 
  cvmSet(box_points, 7, 1,  0.5*dY); 
  cvmSet(box_points, 7, 2,  dZ);

  cvProjectPoints2(box_points, rotation_vector, translation_vector, intrinsic_matrix, distortion_coeffs, image_points);

  for(i=0; i<8; i++)
  {
    p[i].x= (int) cvmGet(image_points, i, 0);
    p[i].y= (int) cvmGet(image_points, i, 1); 
  }

  //draw the box edges:
  color= CV_RGB(0, 255, 255);
  cvLine(image, p[0] , p[1], color,  2, CV_AA, 0);
  cvLine(image, p[1] , p[2], color,  2, CV_AA, 0);
  cvLine(image, p[2] , p[3], color,  2, CV_AA, 0);
  cvLine(image, p[3] , p[0], color,  2, CV_AA, 0);

  cvLine(image, p[4] , p[5], color,  2, CV_AA, 0);
  cvLine(image, p[5] , p[6], color,  2, CV_AA, 0);
  cvLine(image, p[6] , p[7], color,  2, CV_AA, 0);
  cvLine(image, p[7] , p[4], color,  2, CV_AA, 0);

  cvLine(image, p[0] , p[4], color,  2, CV_AA, 0);
  cvLine(image, p[1] , p[5], color,  2, CV_AA, 0);
  cvLine(image, p[2] , p[6], color,  2, CV_AA, 0);
  cvLine(image, p[3] , p[7], color,  2, CV_AA, 0);

  //draw the box points as small circles:
  color= CV_RGB(255, 0, 255);
  for(i=0; i<8; i++)
  {
    cvCircle(image, p[i],  3, color,  0,  CV_AA,  0); 
  }

  cvReleaseMat(&box_points);
  cvReleaseMat(&image_points);
}


// Draw a the image of a frame (three arrows) in the specified image.
// The function receives the camera parameters and the pose of the frame in space (rotation and translation vector).
// The length of the arrows is 'size'.
void vimanDrawFrame(IplImage* image, const CvMat* intrinsic_matrix, const CvMat* distortion_coeffs, CvMat* rotation_vector, CvMat* translation_vector, double size)
{
  int i;
  CvScalar color;
  CvPoint p[4];
  CvMat *frame_points= cvCreateMat( 4, 3, CV_32FC1);
  CvMat *image_points= cvCreateMat( 4, 2, CV_32FC1);

  cvmSet(frame_points, 0, 0,  0.0); 
  cvmSet(frame_points, 0, 1,  0.0); 
  cvmSet(frame_points, 0, 2,  0.0); 

  cvmSet(frame_points, 1, 0, size); 
  cvmSet(frame_points, 1, 1,  0.0); 
  cvmSet(frame_points, 1, 2,  0.0); 

  cvmSet(frame_points, 2, 0,  0.0); 
  cvmSet(frame_points, 2, 1, size); 
  cvmSet(frame_points, 2, 2,  0.0); 

  cvmSet(frame_points, 3, 0,  0.0); 
  cvmSet(frame_points, 3, 1,  0.0); 
  cvmSet(frame_points, 3, 2, size); 


  cvProjectPoints2(frame_points, rotation_vector, translation_vector, intrinsic_matrix, distortion_coeffs, image_points);


  for(i=0; i<4; i++)
  {
    p[i].x= (int) cvmGet(image_points, i, 0);
    p[i].y= (int) cvmGet(image_points, i, 1); 
  }


  color= CV_RGB(255, 0, 0);
  cvLine(image, p[0] , p[1], color,  3, CV_AA, 0);
  cvCircle(image, p[1],  3, color,  0,  CV_AA,  0); 
 
  color= CV_RGB(0, 255, 0);
  cvLine(image, p[0] , p[2], color,  3, CV_AA, 0);
  cvCircle(image, p[2],  3, color,  0,  CV_AA,  0); 

  color= CV_RGB(0, 0, 255);
  cvLine(image, p[0] , p[3], color,  3, CV_AA, 0);
  cvCircle(image, p[3],  3, color,  0,  CV_AA,  0); 


  cvReleaseMat(&frame_points);
  cvReleaseMat(&image_points);
}

// Read the file that contains the list of all the objects (names and .cfg file names).
// The function fills the global array of objects.
int vimanReadObjectListFile(std::string filename)
{
  int length, nbObjects= 0;
  size_t pos;
  std::string str;
  std::ifstream in;


  length= filename.length(); 
  pos= filename.find("_cfg"); 
  if(pos==filename.npos || pos!=length-4)
  {
    printf("VIMAN - ERROR File %s does not have the good extension (it should be a _cfg file).\n", filename.c_str());
    return 0;
  }


  in.open(filename.c_str());

  if(in.fail()==true)
  {
    printf("VIMAN - ERROR File %s was not found.\n", filename.c_str());
    in.close();
    return 0;
  }


  in >> str;
  while(in.eof() == false)
  {
    if(str.compare("object")==0)
    {
      in >>  SDI_F->objectArray.objects[nbObjects].name;
     
      in >> str;
      str.insert(0, "/");
      str.insert(0, PARAM_FILE_PATH);

      strcpy(SDI_F->objectArray.objects[nbObjects].file, str.c_str());

      nbObjects++;
      if(nbObjects >= VIMAN_MAX_OBJ_NB)
      {
      	printf("VIMAN - ERROR Too many objects to store in SDI_F->objectArray.\n");
	      nbObjects--;
	      break;
      }
      
      in >> str;
      continue;
    }
    else
    {
      printf("VIMAN - ERROR syntax error in file %s at place %s \n", filename.c_str(), str.c_str());
      return 0;
    }
  
  } 

  SDI_F->objectArray.nbObjects= nbObjects;
  SDI_F->objectPublicArray.nbObjects= nbObjects;
  printf("There are %d objects \n",  SDI_F->objectPublicArray.nbObjects);

  for(int i=0; i<SDI_F->objectArray.nbObjects; i++)
  {
     printf("object: %s file: %s\n", SDI_F->objectArray.objects[i].name, SDI_F->objectArray.objects[i].file);
  }

  in.close();

  return 1;
}

// This function saves the multimarker configuration file (.cfg) with the format used by ARToolKit
// of the given object once marker calibration is done.
int vimanSaveObjectCfgFile(int object_index)
{
  int i;
  std::string filename;
  std::string fileObject;
  std::ofstream file;

  if(object_index>=SDI_F->objectArray.nbObjects || object_index<0)
  {
    printf("VIMAN - ERROR vimanSaveObjectCfgFile: wrong object index.\n" );
    return 0;
  }

  filename += PARAM_FILE_PATH;
  filename += "/object_cfg/";
  filename += SDI_F->objectArray.objects[object_index].name;
  filename += "_new.cfg";

  file.open(filename.c_str(), std::ios::trunc);

  file << "# marker for object " << SDI_F->objectArray.objects[object_index].name <<std::endl << std::endl;
  file << "# number of markers" <<std::endl;
  file << SDI_F->objectArray.objects[object_index].nbMarkers << std::endl << std::endl << std::endl;
  
  for(i=0; i<SDI_F->objectArray.objects[object_index].nbMarkers; i++)
  {
    file << "# marker ID" <<std::endl;
    file << SDI_F->objectArray.objects[object_index].markerIDs[i] << std::endl;
    file << SDI_F->objectArray.objects[object_index].patternWidths[i] << std::endl;
    file << "0.0 0.0" <<std::endl;

    file << SDI_F->objectArray.objects[object_index].relativePoses[i].nx << ' ' << SDI_F->objectArray.objects[object_index].relativePoses[i].ox << ' ' << SDI_F->objectArray.objects[object_index].relativePoses[i].ax << ' ' << SDI_F->objectArray.objects[object_index].relativePoses[i].px << std::endl;

    file << SDI_F->objectArray.objects[object_index].relativePoses[i].ny << ' ' << SDI_F->objectArray.objects[object_index].relativePoses[i].oy << ' ' << SDI_F->objectArray.objects[object_index].relativePoses[i].ay << ' ' << SDI_F->objectArray.objects[object_index].relativePoses[i].py << std::endl;

    file << SDI_F->objectArray.objects[object_index].relativePoses[i].nz << ' ' << SDI_F->objectArray.objects[object_index].relativePoses[i].oz << ' ' << SDI_F->objectArray.objects[object_index].relativePoses[i].az << ' ' << SDI_F->objectArray.objects[object_index].relativePoses[i].pz << std::endl << std::endl ;
  }
  return 1;
}


//
//int vimanReadVIAMCalibFile(int *l_width, int *l_height, double l_intrinsic_matrix[3][3], double l_distortion[5], int *r_width, int *r_height, double r_intrinsic_matrix[3][3], , double r_distortion[5], std::string filename)
//{
//  int length, line_count;
//  size_t pos;
//  bool has_stereo= false;
//  bool has_baseline= false;
//  bool has_pixel_baseline= false;
//  bool has_camera= false;
//  bool has_size= false;
//  bool has_intrinsic= false;
//  bool has_distortion= false;
//  bool has_row1= false; 
//  char buffer[1024];
//  std::string line, str;
//  std::ifstream in;
//  double baseline, pixel_baseline;
//
//
//  length= filename.length(); 
//  pos= filename.find(".data"); 
//  if(pos==filename.npos || pos!=length-5)
//  {
//    printf("VIMAN - ERROR File %s does not have the good extension (it should be a .data file).\n", filename.c_str());
//    return 0;
//  }    
//
//
//  in.open(filename.c_str());
//
//  if(in.fail()==true)
//  {
//    printf("VIMAN - ERROR File %s was not found.\n", filename.c_str());
//    in.close();
//    return 0;
//  }
//
//  line_count= 0;
//  while(in.eof()==false)
//  {      
//     //Read the file line by line:
//     in.getline(buffer, 512);
//     line_count++;
//     line= buffer;
//
//     //Remove all that's after a '#' character
//     pos= line.find_first_of('#');
//     if(pos!=line.npos)
//     { 
//       if(pos==0) continue;
//       line= line.substr(0, pos);
//     }
//
//     //Remove all the spaces at the end of the line:
//     pos= line.find_last_not_of(' ');
//     line= line.substr(0, pos+1);
//
//     if( line.empty() )
//       continue;
//
//
//     //Create a stream from the line string:
//     iss.clear();
//     iss.str(line);
//
//     if(!has_stereo)
//     { 
//       result= iss >> buffer;
//       if( !result || !iss.eof() )
//       {
//         printf("VIMAN - ERROR in file %s at line %d: %s\n", filename.c_str(), line_count, line.c_str());
//         printf("expected format is: stereo (char *) \n");
//       } 
//       has_stereo= true;
//       continue;
//     }
//
//     if(!has_baseline)
//     { 
//       result= iss >> baseline;
//       if( !result || !iss.eof() )
//       {
//         printf("VIMAN - ERROR in file %s at line %d: %s\n", filename.c_str(), line_count, line.c_str());
//         printf("expected format is: baseline (double) \n");
//       } 
//       has_baseline= true;
//       continue;
//     }
//
//     if(!has_pixel_baseline)
//     { 
//       result= iss >> pixel_baseline;
//       if( !result || !iss.eof() )
//       {
//         printf("VIMAN - ERROR in file %s at line %d: %s\n", filename.c_str(), line_count, line.c_str());
//         printf("expected format is: pixel_baseline (double) \n");
//       } 
//       has_pixel_baseline= true;
//       continue;
//     }
//
//     if(!has_camera)
//     { 
//       result= iss >> buffer;
//       if( !result || !iss.eof() )
//       {
//         printf("VIMAN - ERROR in file %s at line %d: %s\n", filename.c_str(), line_count, line.c_str());
//         printf("expected format is: camera (char *) \n");
//       } 
//       has_nb_markers= true;
//       continue;
//     }
//
//
//     if(!has_size)
//     { 
//       result= iss >> buffer;
//       if( !result || !iss.eof() )
//       {
//         printf("VIMAN - ERROR in file %s at line %d: %s\n", filename.c_str(), line_count, line.c_str());
//         printf("expected format is: size (char *) \n");
//       } 
//       
//       has_nb_size= true;
//       continue;
//     }
//
//
//  }
//
//
//}
//
//
//
// This function reads a multimarker configuration file (.cfg) with the format used by ARToolKit.
// It fills the appropriate fields of the corresponding object in the array of objects.
int vimanReadObjectCfgFile(std::string objectname, std::string filename)
{
  int i, length, line_count, marker_count;
  int object_index;
  int nb_markers, marker_id;
  float pattern_width, center[2];
  float mat[3][4];
  bool result;
  bool has_nb_markers= false;
  bool has_marker_id= false;
  bool has_pattern_width= false;
  bool has_center= false;
  bool has_row1= false; 
  bool has_row2= false; 
  bool has_row3= false; 
  size_t pos;
  char buffer[1024];
  std::string line;
  std::ifstream in;
  std::istringstream iss;


  for(i=0; i<SDI_F->objectArray.nbObjects; i++)
  {
    if(strcmp(SDI_F->objectArray.objects[i].name, objectname.c_str())==0)
    {
      object_index= i;
      break;
    }
  }

  if(i==SDI_F->objectArray.nbObjects)
  {
    printf("VIMAN - ERROR There is no object named %s in the object array.\n", objectname.c_str());
    return 0;
  }


  length= filename.length(); 
  pos= filename.find(".cfg"); 
  if(pos==filename.npos || pos!=length-4)
  {
    printf("VIMAN - ERROR File %s does not have the good extension (it should be a .cfg file).\n", filename.c_str());
    return 0;
  }    


  in.open(filename.c_str());

  if(in.fail()==true)
  {
    printf("VIMAN - ERROR File %s was not found.\n", filename.c_str());
    in.close();
    return 0;
  }

  line_count= 0;
  marker_count= 0;
  while(in.eof()==false)
  {      
     //Read the file line by line:
     in.getline(buffer, 512);
     line_count++;
     line= buffer;
 
     //Remove all that's after a '#' character
     pos= line.find_first_of('#');
     if(pos!=line.npos)
     { 
       if(pos==0) continue;
       line= line.substr(0, pos);
     }

     //Remove all the spaces at the end of the line:
     pos= line.find_last_not_of(' ');
     line= line.substr(0, pos+1);

     if( line.empty() )
       continue;

     //Create a stream from the line string:
     iss.clear();
     iss.str(line);

     if(!has_nb_markers)
     { 
       result= iss >> nb_markers;
       if( !result || !iss.eof() )
       {
         printf("VIMAN - ERROR in file %s at line %d: %s\n", filename.c_str(), line_count, line.c_str());
         printf("expected format is: nb_markers (int) \n");
       } 
       has_nb_markers= true;
       continue;
     }

     if(!has_marker_id)
     { 
       result= iss >> marker_id;
       if( !result || !iss.eof() )
       { 
         printf("VIMAN - ERROR in file %s at line %d: %s\n", filename.c_str(), line_count, line.c_str());
         printf("expected format is: marker_id (int) \n");
       } 
       has_marker_id= true;
       continue;
     }

     if(!has_pattern_width)
     { 
       result= iss >> pattern_width;
       if( !result || !iss.eof() )
       {
         printf("VIMAN - ERROR in file %s at line %d: %s\n", filename.c_str(), line_count, line.c_str());
         printf("expected format is: pattern_width (float) \n");
       } 
       has_pattern_width= true;
       continue;
     }

     if(!has_center)
     { 
       result= iss >> center[0] >> center[1];
       if( !result || !iss.eof() )
       {
         printf("VIMAN - ERROR in file %s at line %d: %s\n", filename.c_str(), line_count, line.c_str());
         printf("expected format is: centerX centerY (float float) \n");
       } 
       has_center= true;
       continue;
     }

     if(!has_row1)
     { 
       result= iss >> mat[0][0] >> mat[0][1] >> mat[0][2] >> mat[0][3];
       if( !result || !iss.eof() )
       {
         printf("VIMAN - ERROR in file %s at line %d: %s\n", filename.c_str(), line_count, line.c_str());
         printf("expected format is: matrix[0][0] >> matrix[0][1] >> matrix[0][2] >> matrix[0][3] (float float float float) \n");
       } 
       has_row1= true;
       continue;
     }

     if(!has_row2)
     { 
       result= iss >> mat[1][0] >> mat[1][1] >> mat[1][2] >> mat[1][3];
       if( !result || !iss.eof() )
       {
         printf("VIMAN - ERROR in file %s at line %d: %s\n", filename.c_str(), line_count, line.c_str());
         printf("expected format is: matrix[1][0] >> matrix[1][1] >> matrix[1][2] >> matrix[1][3] (float float float float) \n");
       } 
       has_row2= true;
       continue;
     }

     if(!has_row3)
     { 
       result= iss >> mat[2][0] >> mat[2][1] >> mat[2][2] >> mat[2][3];
       if( !result || !iss.eof() )
       {
         printf("VIMAN - ERROR in file %s at line %d: %s\n", filename.c_str(), line_count, line.c_str());
         printf("expected format is: matrix[2][0] >> matrix[2][1] >> matrix[2][2] >> matrix[2][3] (float float float float) \n");
       } 
       has_row3= true;
       
       SDI_F->objectArray.objects[object_index].nbMarkers= marker_count + 1;
       SDI_F->objectArray.objects[object_index].markerIDs[marker_count]= marker_id;
       SDI_F->objectArray.objects[object_index].patternWidths[marker_count]= pattern_width;

       SDI_F->objectArray.objects[object_index].relativePoses[marker_count].nx= mat[0][0];
       SDI_F->objectArray.objects[object_index].relativePoses[marker_count].ny= mat[1][0];
       SDI_F->objectArray.objects[object_index].relativePoses[marker_count].nz= mat[2][0];

       SDI_F->objectArray.objects[object_index].relativePoses[marker_count].ox= mat[0][1];
       SDI_F->objectArray.objects[object_index].relativePoses[marker_count].oy= mat[1][1];
       SDI_F->objectArray.objects[object_index].relativePoses[marker_count].oz= mat[2][1];

       SDI_F->objectArray.objects[object_index].relativePoses[marker_count].ax= mat[0][2];
       SDI_F->objectArray.objects[object_index].relativePoses[marker_count].ay= mat[1][2];
       SDI_F->objectArray.objects[object_index].relativePoses[marker_count].az= mat[2][2];

       SDI_F->objectArray.objects[object_index].relativePoses[marker_count].px= mat[0][3];
       SDI_F->objectArray.objects[object_index].relativePoses[marker_count].py= mat[1][3];
       SDI_F->objectArray.objects[object_index].relativePoses[marker_count].pz= mat[2][3];

       marker_count++;

       has_marker_id= false;
       has_pattern_width= false;
       has_center= false;
       has_row1= false; 
       has_row2= false; 
       has_row3= false; 
     }

  }


  in.close();

  return 1;
}


// Return 1 if the object of index 'object_index' in the object array has a marker with the ID 'marker_id'.
// Return 0 otherwise. If the object has a marker with the given ID, the index of this marker in the marker
// array of the object is copied in 'marker_index'.
int vimanFindMarkerIndex(int object_index, int marker_id, int *marker_index)
{
  int i;
  
  for(i=0; i<SDI_F->objectArray.objects[object_index].nbMarkers; i++)
  {
    if(SDI_F->objectArray.objects[object_index].markerIDs[i]==marker_id)
    {
      *marker_index= i;
      return 1;
    }
  }


  return 0;
}

// Computes the pose wrt camera of the marker whose corners positions in the image are in the 'marker_corners' array
// and should be in the good ordering (i.e. correspond to (0,0), (0,1), (1,1), (1,0) positions in the marker frame).
// The function receives also the camera parameters and the pattern width of the marker.
// The pose is copied in 'rotation_vector' and 'translation_vector'.
// In case of success, the function returns 1, it returns 0 otherwise.
int vimanFindMarkerPoseInCameraFrame(const CvMat *intrinsic_matrix, const CvMat *dist_coeffs, CvPoint2D32f marker_corners[4], double pattern_width, CvMat *rotation_vector, CvMat *translation_vector)
{
  if( !CV_IS_MAT(intrinsic_matrix) || intrinsic_matrix->rows != 3 || intrinsic_matrix->cols != 3  ||
      (CV_MAT_TYPE(intrinsic_matrix->type) != CV_32FC1 && CV_MAT_TYPE(intrinsic_matrix->type) != CV_64FC1) )
  {
    printf("VIMAN - ERROR vimanFindMarkerPoseInCameraFrame: Intrinsic matrix must be a valid 3x3 floating-point matrix.\n" );
    return 0;
  }
  if( !CV_IS_MAT(rotation_vector) || rotation_vector->rows != 3 || rotation_vector->cols != 1 ||
      (CV_MAT_TYPE(rotation_vector->type) != CV_32FC1 && CV_MAT_TYPE(rotation_vector->type) != CV_64FC1) )
  {
    printf("VIMAN - ERROR vimanFindMarkerPoseInCameraFrame: Rotation vector a valid 3x1 or 1x3 floating-point matrix.\n" );
    return 0;
  }
  if( !CV_IS_MAT(translation_vector) || translation_vector->rows != 3 || translation_vector->cols != 1 ||
      (CV_MAT_TYPE(translation_vector->type) != CV_32FC1 && CV_MAT_TYPE(translation_vector->type) != CV_64FC1) )
  {
    printf("VIMAN - ERROR vimanFindMarkerPoseInCameraFrame: Translation vector a valid 3x1 or 1x3 floating-point matrix.\n" );
    return 0;
  }


  int i, j;
  CvMat *object_points  = cvCreateMat( 4, 3, CV_32FC1);
  CvMat *image_points   = cvCreateMat( 4, 2, CV_32FC1);

  //Fill the marker corners 3D positions:
  cvmSet(object_points, 0, 0, -0.5*pattern_width); 
  cvmSet(object_points, 0, 1, -0.5*pattern_width); 
  cvmSet(object_points, 0, 2,  0.0); 

  cvmSet(object_points, 1, 0, -0.5*pattern_width); 
  cvmSet(object_points, 1, 1,  0.5*pattern_width); 
  cvmSet(object_points, 1, 2,  0.0); 

  cvmSet(object_points, 2, 0,  0.5*pattern_width); 
  cvmSet(object_points, 2, 1,  0.5*pattern_width); 
  cvmSet(object_points, 2, 2,  0.0); 

  cvmSet(object_points, 3, 0,  0.5*pattern_width); 
  cvmSet(object_points, 3, 1, -0.5*pattern_width); 
  cvmSet(object_points, 3, 2,  0.0); 

  //Fill the marker corners image positions:
  cvmSet(image_points, 0, 0, marker_corners[0].x); 
  cvmSet(image_points, 0, 1, marker_corners[0].y);

  cvmSet(image_points, 1, 0, marker_corners[1].x); 
  cvmSet(image_points, 1, 1, marker_corners[1].y);

  cvmSet(image_points, 2, 0, marker_corners[2].x); 
  cvmSet(image_points, 2, 1, marker_corners[2].y);

  cvmSet(image_points, 3, 0, marker_corners[3].x); 
  cvmSet(image_points, 3, 1, marker_corners[3].y);


  //Compute the marker pose wrt camera frame:
  cvFindExtrinsicCameraParams2(object_points, image_points, intrinsic_matrix, dist_coeffs, rotation_vector, translation_vector);

  cvReleaseMat(&image_points);
  cvReleaseMat(&object_points); 

  return 1;
}


// Return the number of marker that have been calibrated for the object of index 'object_index'
// in the global object array.
int vimanCountCalibratedMarkers(int object_index)
{
  int i, count= 0;

  for(i=0; i<SDI_F->objectArray.objects[object_index].nbMarkers; i++)
  {
    if(SDI_F->objectArray.objects[object_index].markerCalibrated[i]==1)
      count++;
  }
  
  return count;
}

// Return 1 if all the markers of the object have been calibrated, 0 otherwise.
int vimanAreAllMarkersCalibrated(int object_index)
{
  int i;

  for(i=0; i<SDI_F->objectArray.objects[object_index].nbMarkers; i++)
  {
    if(SDI_F->objectArray.objects[object_index].markerCalibrated[i]==0)
      return 0;
  }
  
  return 1;
}

//
//// For the object of index 'object_index', calibrates the marker of index 'marker_index' (computes 
//// the pose of the marker wrt the object)
//// The function receives the marker pose wrt camera ('rotation_vector' and 'translation_vector').
//// It uses the pose of the baseFrame object (that must be defined in the object list file and have been detected)
//// in the global object array.
//// The function then fills (overwrites) the relative marker/object pose array (in the object structure). 
//// In case of success, the function returns 1, it returns 0 otherwise.
//int vimanCalibrateMarker(int object_index, int marker_index, CvMat *rotation_vector, CvMat *translation_vector)
//{
//  if( !CV_IS_MAT(rotation_vector) || rotation_vector->rows != 3 || rotation_vector->cols != 1 ||
//      (CV_MAT_TYPE(rotation_vector->type) != CV_32FC1 && CV_MAT_TYPE(rotation_vector->type) != CV_64FC1) )
//  {
//    printf("VIMAN - ERROR vimanCalibrateMarker: Rotation vector a valid 3x1 or 1x3 floating-point matrix.\n" );
//    return 0;
//  }
//  if( !CV_IS_MAT(translation_vector) || translation_vector->rows != 3 || translation_vector->cols != 1 ||
//      (CV_MAT_TYPE(translation_vector->type) != CV_32FC1 && CV_MAT_TYPE(translation_vector->type) != CV_64FC1) )
//  {
//    printf("VIMAN - ERROR vimanCalibrateMarker: Translation vector a valid 3x1 or 1x3 floating-point matrix.\n" );
//    return 0;
//  }
//  if(object_index>=SDI_F->objectArray.nbObjects || object_index<0)
//  {
//    printf("VIMAN - ERROR vimanCalibrateMarker: wrong object index.\n" );
//    return 0;
//  }
//  if(marker_index>=SDI_F->objectArray.objects[object_index].nbMarkers || marker_index<0)
//  {
//    printf("VIMAN - ERROR vimanCalibrateMarker: wrong marker index.\n" );
//    return 0;
//  }
//
//  int i, j;
//  CvMat *rotation_matrix   = cvCreateMat( 3, 3, CV_32FC1);
//  CvMat *Tmarker_camera    = cvCreateMat( 4, 4, CV_32FC1);
//  CvMat *TbaseFrame_camera = cvCreateMat( 4, 4, CV_32FC1); //->= Tobject_camera
//  CvMat *Tmarker_object    = cvCreateMat( 4, 4, CV_32FC1);
//  CvMat *Tcamera_object    = cvCreateMat( 4, 4, CV_32FC1);
//
//
//  for(i=0; i<SDI_F->objectArray.nbObjects; i++)
//  {
//    if( strcmp(SDI_F->objectArray.objects[i].name,"baseFrame")==0 )
//    {
//      if(SDI_F->objectArray.objects[i].found_L==0)
//      {
//        printf("VIMAN - ERROR vimanCalibrateMarker: base frame has not been detected.\n" );
//        cvReleaseMat(&rotation_matrix); 
//        cvReleaseMat(&Tmarker_camera); 
//        cvReleaseMat(&TbaseFrame_camera); 
//        cvReleaseMat(&Tmarker_object); 
//        cvReleaseMat(&Tcamera_object);
//        return 0;
//      }    
//
//
//      cvmSet(TbaseFrame_camera, 0, 0,  SDI_F->objectArray.objects[i].thetaMatCam.nx); 
//      cvmSet(TbaseFrame_camera, 1, 0,  SDI_F->objectArray.objects[i].thetaMatCam.ny); 
//      cvmSet(TbaseFrame_camera, 2, 0,  SDI_F->objectArray.objects[i].thetaMatCam.nz); 
//      cvmSet(TbaseFrame_camera, 3, 0,  0); 				     
//									     
//      cvmSet(TbaseFrame_camera, 0, 1,  SDI_F->objectArray.objects[i].thetaMatCam.ox); 
//      cvmSet(TbaseFrame_camera, 1, 1,  SDI_F->objectArray.objects[i].thetaMatCam.oy); 
//      cvmSet(TbaseFrame_camera, 2, 1,  SDI_F->objectArray.objects[i].thetaMatCam.oz); 
//      cvmSet(TbaseFrame_camera, 3, 1,  0); 				     
//									     
//      cvmSet(TbaseFrame_camera, 0, 2,  SDI_F->objectArray.objects[i].thetaMatCam.ax); 
//      cvmSet(TbaseFrame_camera, 1, 2,  SDI_F->objectArray.objects[i].thetaMatCam.ay); 
//      cvmSet(TbaseFrame_camera, 2, 2,  SDI_F->objectArray.objects[i].thetaMatCam.az); 
//      cvmSet(TbaseFrame_camera, 3, 2,  0); 				     
//									     
//      cvmSet(TbaseFrame_camera, 0, 3,  SDI_F->objectArray.objects[i].thetaMatCam.px); 
//      cvmSet(TbaseFrame_camera, 1, 3,  SDI_F->objectArray.objects[i].thetaMatCam.py); 
//      cvmSet(TbaseFrame_camera, 2, 3,  SDI_F->objectArray.objects[i].thetaMatCam.pz); 
//      cvmSet(TbaseFrame_camera, 3, 3,  1); 
//    }
//  }
//
//  cvRodrigues2(rotation_vector, rotation_matrix);
//  for(i=0; i<3; i++)
//  {
//    for(j=0; j<3; j++)
//    {
//      cvmSet(Tmarker_camera, i, j, cvmGet(rotation_matrix, i, j) ); 
//    }
//  }
//  cvmSet(Tmarker_camera, 3, 0, 0); 
//  cvmSet(Tmarker_camera, 3, 1, 0); 
//  cvmSet(Tmarker_camera, 3, 2, 0); 
//  cvmSet(Tmarker_camera, 3, 3, 1); 
//
//  cvmSet(Tmarker_camera, 0, 3,  cvmGet(translation_vector, 0, 0) ); 
//  cvmSet(Tmarker_camera, 1, 3,  cvmGet(translation_vector, 1, 0) ); 
//  cvmSet(Tmarker_camera, 2, 3,  cvmGet(translation_vector, 2, 0) ); 
//
//  cvInvert(TbaseFrame_camera, Tcamera_object, CV_LU);  
//
//  //Compute Tmarker_object= Tcamera_object*Tmarker_camera
//  cvGEMM(Tcamera_object, Tmarker_camera, 1.0, NULL, 0.0, Tmarker_object);
//
//
//  SDI_F->objectArray.objects[object_index].relativePoses[marker_index].nx= cvmGet(Tmarker_object, 0, 0);
//  SDI_F->objectArray.objects[object_index].relativePoses[marker_index].ny= cvmGet(Tmarker_object, 1, 0);
//  SDI_F->objectArray.objects[object_index].relativePoses[marker_index].nz= cvmGet(Tmarker_object, 2, 0);
// 
//  SDI_F->objectArray.objects[object_index].relativePoses[marker_index].ox= cvmGet(Tmarker_object, 0, 1);
//  SDI_F->objectArray.objects[object_index].relativePoses[marker_index].oy= cvmGet(Tmarker_object, 1, 1);
//  SDI_F->objectArray.objects[object_index].relativePoses[marker_index].oz= cvmGet(Tmarker_object, 2, 1);
//
//  SDI_F->objectArray.objects[object_index].relativePoses[marker_index].ax= cvmGet(Tmarker_object, 0, 2);
//  SDI_F->objectArray.objects[object_index].relativePoses[marker_index].ay= cvmGet(Tmarker_object, 1, 2);
//  SDI_F->objectArray.objects[object_index].relativePoses[marker_index].az= cvmGet(Tmarker_object, 2, 2);
//
//  SDI_F->objectArray.objects[object_index].relativePoses[marker_index].px= cvmGet(Tmarker_object, 0, 3);
//  SDI_F->objectArray.objects[object_index].relativePoses[marker_index].py= cvmGet(Tmarker_object, 1, 3);
//  SDI_F->objectArray.objects[object_index].relativePoses[marker_index].pz= cvmGet(Tmarker_object, 2, 3);  
//
//
//  cvReleaseMat(&rotation_matrix); 
//  cvReleaseMat(&Tmarker_camera); 
//  cvReleaseMat(&TbaseFrame_camera); 
//  cvReleaseMat(&Tmarker_object); 
//  cvReleaseMat(&Tcamera_object);
//
//  return 1;
//}


// Computes the pose wrt camera of the object of index 'object_index' in the global object array, using
// the pose of the marker with index 'marker_index' in the marker array of the object.
// The pose of the marker is in 'rotation_vector' and 'translation_vector'.
// The object pose is computed using the marker/object pose parameters 
// in the VimanObject structure and is overwritten in 'rotation_vector' and 'translation_vector'.
// In case of success, the function returns 1, it returns 0 otherwise.
int vimanFindObjectPoseInCameraFrame(int object_index, int marker_index, CvMat *rotation_vector, CvMat *translation_vector)
{
  if( !CV_IS_MAT(rotation_vector) || rotation_vector->rows != 3 || rotation_vector->cols != 1 ||
      (CV_MAT_TYPE(rotation_vector->type) != CV_32FC1 && CV_MAT_TYPE(rotation_vector->type) != CV_64FC1) )
  {
    printf("VIMAN - ERROR vimanFindObjectPoseInCameraFrame: Rotation vector a valid 3x1 or 1x3 floating-point matrix.\n" );
    return 0;
  }
  if( !CV_IS_MAT(translation_vector) || translation_vector->rows != 3 || translation_vector->cols != 1 ||
      (CV_MAT_TYPE(translation_vector->type) != CV_32FC1 && CV_MAT_TYPE(translation_vector->type) != CV_64FC1) )
  {
    printf("VIMAN - ERROR vimanFindObjectPoseInCameraFrame: Translation vector a valid 3x1 or 1x3 floating-point matrix.\n" );
    return 0;
  }
  if(object_index>=SDI_F->objectArray.nbObjects || object_index<0)
  {
    printf("VIMAN - ERROR vimanFindObjectPoseInCameraFrame: wrong object index.\n" );
    return 0;
  }
  if(marker_index>=SDI_F->objectArray.objects[object_index].nbMarkers || marker_index<0)
  {
    printf("VIMAN - ERROR vimanFindObjectPoseInCameraFrame: wrong marker index.\n" );
    return 0;
  }

  int i, j;

  CvMat *rotation_matrix= cvCreateMat( 3, 3, CV_32FC1);
  CvMat *Tmarker_camera = cvCreateMat( 4, 4, CV_32FC1);
  CvMat *Tmarker_object = cvCreateMat( 4, 4, CV_32FC1);
  CvMat *Tobject_marker = cvCreateMat( 4, 4, CV_32FC1);
  CvMat *Tobject_camera = cvCreateMat( 4, 4, CV_32FC1);

  //Compute the object pose wrt camera frame:
  cvRodrigues2(rotation_vector, rotation_matrix);
  for(i=0; i<3; i++)
  {
    for(j=0; j<3; j++)
    {
      cvmSet(Tmarker_camera, i, j, cvmGet(rotation_matrix, i, j) ); 
    }
  }
  cvmSet(Tmarker_camera, 3, 0, 0); 
  cvmSet(Tmarker_camera, 3, 1, 0); 
  cvmSet(Tmarker_camera, 3, 2, 0); 
  cvmSet(Tmarker_camera, 3, 3, 1); 

  cvmSet(Tmarker_camera, 0, 3,  cvmGet(translation_vector, 0, 0) ); 
  cvmSet(Tmarker_camera, 1, 3,  cvmGet(translation_vector, 1, 0) ); 
  cvmSet(Tmarker_camera, 2, 3,  cvmGet(translation_vector, 2, 0) ); 

  cvmSet(Tmarker_object, 0, 0,  SDI_F->objectArray.objects[object_index].relativePoses[marker_index].nx); 
  cvmSet(Tmarker_object, 1, 0,  SDI_F->objectArray.objects[object_index].relativePoses[marker_index].ny); 
  cvmSet(Tmarker_object, 2, 0,  SDI_F->objectArray.objects[object_index].relativePoses[marker_index].nz); 
  cvmSet(Tmarker_object, 3, 0,  0); 

  cvmSet(Tmarker_object, 0, 1,  SDI_F->objectArray.objects[object_index].relativePoses[marker_index].ox); 
  cvmSet(Tmarker_object, 1, 1,  SDI_F->objectArray.objects[object_index].relativePoses[marker_index].oy); 
  cvmSet(Tmarker_object, 2, 1,  SDI_F->objectArray.objects[object_index].relativePoses[marker_index].oz); 
  cvmSet(Tmarker_object, 3, 1,  0); 

  cvmSet(Tmarker_object, 0, 2,  SDI_F->objectArray.objects[object_index].relativePoses[marker_index].ax); 
  cvmSet(Tmarker_object, 1, 2,  SDI_F->objectArray.objects[object_index].relativePoses[marker_index].ay); 
  cvmSet(Tmarker_object, 2, 2,  SDI_F->objectArray.objects[object_index].relativePoses[marker_index].az); 
  cvmSet(Tmarker_object, 3, 2,  0); 

  cvmSet(Tmarker_object, 0, 3,  SDI_F->objectArray.objects[object_index].relativePoses[marker_index].px); 
  cvmSet(Tmarker_object, 1, 3,  SDI_F->objectArray.objects[object_index].relativePoses[marker_index].py); 
  cvmSet(Tmarker_object, 2, 3,  SDI_F->objectArray.objects[object_index].relativePoses[marker_index].pz); 
  cvmSet(Tmarker_object, 3, 3,  1); 

  cvInvert(Tmarker_object, Tobject_marker, CV_LU);  

  //Compute Tobject_camera= Tmarker_camera*Tobject_marker
  cvGEMM(Tmarker_camera, Tobject_marker, 1.0, NULL, 0.0, Tobject_camera);


  SDI_F->objectArray.objects[object_index].thetaMatCam.nx= cvmGet(Tobject_camera, 0, 0);
  SDI_F->objectArray.objects[object_index].thetaMatCam.ny= cvmGet(Tobject_camera, 1, 0);
  SDI_F->objectArray.objects[object_index].thetaMatCam.nz= cvmGet(Tobject_camera, 2, 0);
 
  SDI_F->objectArray.objects[object_index].thetaMatCam.ox= cvmGet(Tobject_camera, 0, 1);
  SDI_F->objectArray.objects[object_index].thetaMatCam.oy= cvmGet(Tobject_camera, 1, 1);
  SDI_F->objectArray.objects[object_index].thetaMatCam.oz= cvmGet(Tobject_camera, 2, 1);

  SDI_F->objectArray.objects[object_index].thetaMatCam.ax= cvmGet(Tobject_camera, 0, 2);
  SDI_F->objectArray.objects[object_index].thetaMatCam.ay= cvmGet(Tobject_camera, 1, 2);
  SDI_F->objectArray.objects[object_index].thetaMatCam.az= cvmGet(Tobject_camera, 2, 2);

  SDI_F->objectArray.objects[object_index].thetaMatCam.px= cvmGet(Tobject_camera, 0, 3);
  SDI_F->objectArray.objects[object_index].thetaMatCam.py= cvmGet(Tobject_camera, 1, 3);
  SDI_F->objectArray.objects[object_index].thetaMatCam.pz= cvmGet(Tobject_camera, 2, 3);  

  // Write to the public poster

  SDI_F->objectPublicArray.objects[object_index].thetaMatCam.nx= cvmGet(Tobject_camera, 0, 0);
  SDI_F->objectPublicArray.objects[object_index].thetaMatCam.ny= cvmGet(Tobject_camera, 1, 0);
  SDI_F->objectPublicArray.objects[object_index].thetaMatCam.nz= cvmGet(Tobject_camera, 2, 0);
 
  SDI_F->objectPublicArray.objects[object_index].thetaMatCam.ox= cvmGet(Tobject_camera, 0, 1);
  SDI_F->objectPublicArray.objects[object_index].thetaMatCam.oy= cvmGet(Tobject_camera, 1, 1);
  SDI_F->objectPublicArray.objects[object_index].thetaMatCam.oz= cvmGet(Tobject_camera, 2, 1);

  SDI_F->objectPublicArray.objects[object_index].thetaMatCam.ax= cvmGet(Tobject_camera, 0, 2);
  SDI_F->objectPublicArray.objects[object_index].thetaMatCam.ay= cvmGet(Tobject_camera, 1, 2);
  SDI_F->objectPublicArray.objects[object_index].thetaMatCam.az= cvmGet(Tobject_camera, 2, 2);

  SDI_F->objectPublicArray.objects[object_index].thetaMatCam.px= cvmGet(Tobject_camera, 0, 3)/1000;
  SDI_F->objectPublicArray.objects[object_index].thetaMatCam.py= cvmGet(Tobject_camera, 1, 3)/1000;
  SDI_F->objectPublicArray.objects[object_index].thetaMatCam.pz= cvmGet(Tobject_camera, 2, 3)/1000;   


  for(i=0; i<3; i++)
  {
    for(j=0; j<3; j++)
    {
      cvmSet(rotation_matrix, i, j, cvmGet(Tobject_camera, i, j) ); 
    }
  }
  cvRodrigues2(rotation_matrix, rotation_vector);
  cvmSet(translation_vector, 0, 0, cvmGet(Tobject_camera, 0, 3) ); 
  cvmSet(translation_vector, 1, 0, cvmGet(Tobject_camera, 1, 3) ); 
  cvmSet(translation_vector, 2, 0, cvmGet(Tobject_camera, 2, 3) ); 

  cvReleaseMat(&rotation_matrix); 
  cvReleaseMat(&Tmarker_camera); 
  cvReleaseMat(&Tmarker_object); 
  cvReleaseMat(&Tobject_marker); 
  cvReleaseMat(&Tobject_camera);

  return 1;
}



int vimanFindObjectPoseInRobotFrame(int object_index, T3D t3dCam)
{
  T3D t3dObjCam;
  T3D t3dObjRob;
  T3D t3dcamArt;
  T3D t3dObjCamPom;
  T3D t3dcamRob;
  T3D t3dObjRob30;

  /* homogeneous matrix transformation between pomCameraFrame and ARToolKitCameraFrame */
  t3dcamArt.matrix[3] = 0.0;
  t3dcamArt.matrix[7] = 0.0;
  t3dcamArt.matrix[11] = 0.0;
  t3dcamArt.matrix[15] = 1.0;
  t3dcamArt.matrix[0] = 0.0;
  t3dcamArt.matrix[4] = -1.0;
  t3dcamArt.matrix[8] = 0.0;
  t3dcamArt.matrix[1] = 0.0;
  t3dcamArt.matrix[5] = 0.0;
  t3dcamArt.matrix[9] = -1.0;
  t3dcamArt.matrix[2] = 1.0;
  t3dcamArt.matrix[6] = 0.0;
  t3dcamArt.matrix[10] = 0.0;
  t3dcamArt.matrix[12] = 0.0;
  t3dcamArt.matrix[13] = 0.0;
  t3dcamArt.matrix[14] = 0.0;

  /* homogeneous matrix transformation between robot frame and pomCameraFrame */
  t3dcamRob.matrix[0]  = SDI_F->CAMERA_LEFT_ROB_POSE.nx;
  t3dcamRob.matrix[1]  = SDI_F->CAMERA_LEFT_ROB_POSE.ox;
  t3dcamRob.matrix[2]  = SDI_F->CAMERA_LEFT_ROB_POSE.ax;
  t3dcamRob.matrix[3]  = SDI_F->CAMERA_LEFT_ROB_POSE.px;
  t3dcamRob.matrix[4]  = SDI_F->CAMERA_LEFT_ROB_POSE.ny;
  t3dcamRob.matrix[5]  = SDI_F->CAMERA_LEFT_ROB_POSE.oy;
  t3dcamRob.matrix[6]  = SDI_F->CAMERA_LEFT_ROB_POSE.ay;
  t3dcamRob.matrix[7]  = SDI_F->CAMERA_LEFT_ROB_POSE.py;
  t3dcamRob.matrix[8]  = SDI_F->CAMERA_LEFT_ROB_POSE.nz;
  t3dcamRob.matrix[9]  = SDI_F->CAMERA_LEFT_ROB_POSE.oz;
  t3dcamRob.matrix[10] = SDI_F->CAMERA_LEFT_ROB_POSE.az;
  t3dcamRob.matrix[11] = SDI_F->CAMERA_LEFT_ROB_POSE.pz;
  t3dcamRob.matrix[12] = 0.0;
  t3dcamRob.matrix[13] = 0.0;
  t3dcamRob.matrix[14] = 0.0;
  t3dcamRob.matrix[15] = 1.0;

  t3dObjCam.matrix[0]  = SDI_F->objectArray.objects[object_index].thetaMatCam.nx;
  t3dObjCam.matrix[4]  = SDI_F->objectArray.objects[object_index].thetaMatCam.ny;
  t3dObjCam.matrix[8]  = SDI_F->objectArray.objects[object_index].thetaMatCam.nz;
  t3dObjCam.matrix[1]  = SDI_F->objectArray.objects[object_index].thetaMatCam.ox;
  t3dObjCam.matrix[5]  = SDI_F->objectArray.objects[object_index].thetaMatCam.oy;
  t3dObjCam.matrix[9]  = SDI_F->objectArray.objects[object_index].thetaMatCam.oz;
  t3dObjCam.matrix[2]  = SDI_F->objectArray.objects[object_index].thetaMatCam.ax;
  t3dObjCam.matrix[6]  = SDI_F->objectArray.objects[object_index].thetaMatCam.ay;
  t3dObjCam.matrix[10] = SDI_F->objectArray.objects[object_index].thetaMatCam.az;
  t3dObjCam.matrix[3]  = SDI_F->objectArray.objects[object_index].thetaMatCam.px / 1000.0;
  t3dObjCam.matrix[7]  = SDI_F->objectArray.objects[object_index].thetaMatCam.py / 1000.0;
  t3dObjCam.matrix[11] = SDI_F->objectArray.objects[object_index].thetaMatCam.pz / 1000.0;
  t3dObjCam.matrix[12] = 0.0;
  t3dObjCam.matrix[13] = 0.0;
  t3dObjCam.matrix[14] = 0.0;
  t3dObjCam.matrix[15] = 1.0;

  //  printf("Xcam %f Ycam %f Zcam %f \n", t3dObjCam.matrix[3],  t3dObjCam.matrix[7], t3dObjCam.matrix[11]);

  /* compute object pose in the robot frame*/
  t3dObjRob =  (t3dCam * t3dcamArt) * t3dObjCam;
  t3dObjCamPom = t3dcamArt * t3dObjCam;
  t3dObjRob30 =  (t3dcamRob * t3dcamArt) * t3dObjCam;

 
  SDI_F->objectPublicArray.objects[object_index].thetaMatCam.nx =  t3dObjCamPom.matrix[0];
  SDI_F->objectPublicArray.objects[object_index].thetaMatCam.ny =  t3dObjCamPom.matrix[4];
  SDI_F->objectPublicArray.objects[object_index].thetaMatCam.nz =  t3dObjCamPom.matrix[8];				  
  SDI_F->objectPublicArray.objects[object_index].thetaMatCam.ox =  t3dObjCamPom.matrix[1];
  SDI_F->objectPublicArray.objects[object_index].thetaMatCam.oy =  t3dObjCamPom.matrix[5];
  SDI_F->objectPublicArray.objects[object_index].thetaMatCam.oz =  t3dObjCamPom.matrix[9];
  SDI_F->objectPublicArray.objects[object_index].thetaMatCam.ax =  t3dObjCamPom.matrix[2];
  SDI_F->objectPublicArray.objects[object_index].thetaMatCam.ay =  t3dObjCamPom.matrix[6];
  SDI_F->objectPublicArray.objects[object_index].thetaMatCam.az =  t3dObjCamPom.matrix[10];
  SDI_F->objectPublicArray.objects[object_index].thetaMatCam.px =  t3dObjCamPom.matrix[3];
  SDI_F->objectPublicArray.objects[object_index].thetaMatCam.py =  t3dObjCamPom.matrix[7];
  SDI_F->objectPublicArray.objects[object_index].thetaMatCam.pz =  t3dObjCamPom.matrix[11]; 

  // printf("Xcam %f Ycam %f Zcam %f \n", t3dObjRob.matrix[3],  t3dObjRob.matrix[7], t3dObjRob.matrix[11]);
	  
  SDI_F->objectArray.objects[object_index].thetaMatRob.nx =  t3dObjRob.matrix[0];
  SDI_F->objectArray.objects[object_index].thetaMatRob.ny =  t3dObjRob.matrix[4];
  SDI_F->objectArray.objects[object_index].thetaMatRob.nz =  t3dObjRob.matrix[8];				  
  SDI_F->objectArray.objects[object_index].thetaMatRob.ox =  t3dObjRob.matrix[1];
  SDI_F->objectArray.objects[object_index].thetaMatRob.oy =  t3dObjRob.matrix[5];
  SDI_F->objectArray.objects[object_index].thetaMatRob.oz =  t3dObjRob.matrix[9];
  SDI_F->objectArray.objects[object_index].thetaMatRob.ax =  t3dObjRob.matrix[2];
  SDI_F->objectArray.objects[object_index].thetaMatRob.ay =  t3dObjRob.matrix[6];
  SDI_F->objectArray.objects[object_index].thetaMatRob.az =  t3dObjRob.matrix[10];
  SDI_F->objectArray.objects[object_index].thetaMatRob.px =  t3dObjRob.matrix[3];
  SDI_F->objectArray.objects[object_index].thetaMatRob.py =  t3dObjRob.matrix[7];
  SDI_F->objectArray.objects[object_index].thetaMatRob.pz =  t3dObjRob.matrix[11];  

  SDI_F->objectArray.objects[object_index].thetaMatRob30.nx =  t3dObjRob30.matrix[0];
  SDI_F->objectArray.objects[object_index].thetaMatRob30.ny =  t3dObjRob30.matrix[4];
  SDI_F->objectArray.objects[object_index].thetaMatRob30.nz =  t3dObjRob30.matrix[8];				  
  SDI_F->objectArray.objects[object_index].thetaMatRob30.ox =  t3dObjRob30.matrix[1];
  SDI_F->objectArray.objects[object_index].thetaMatRob30.oy =  t3dObjRob30.matrix[5];
  SDI_F->objectArray.objects[object_index].thetaMatRob30.oz =  t3dObjRob30.matrix[9];
  SDI_F->objectArray.objects[object_index].thetaMatRob30.ax =  t3dObjRob30.matrix[2];
  SDI_F->objectArray.objects[object_index].thetaMatRob30.ay =  t3dObjRob30.matrix[6];
  SDI_F->objectArray.objects[object_index].thetaMatRob30.az =  t3dObjRob30.matrix[10];
  SDI_F->objectArray.objects[object_index].thetaMatRob30.px =  t3dObjRob30.matrix[3];
  SDI_F->objectArray.objects[object_index].thetaMatRob30.py =  t3dObjRob30.matrix[7];
  SDI_F->objectArray.objects[object_index].thetaMatRob30.pz =  t3dObjRob30.matrix[11]; 
  
  return 1;
}



int vimanFindObjectPoseInOriginFrame(int object_index, T3D t3dMainToOrigin)
{
  T3D t3dObjToOrigin;
  T3D t3dObjRob;
  T3D t3dObjRob30;
  T3D t3dObjToOrigin30;

  t3dObjRob.matrix[0]  = SDI_F->objectArray.objects[object_index].thetaMatRob.nx;
  t3dObjRob.matrix[4]  = SDI_F->objectArray.objects[object_index].thetaMatRob.ny;
  t3dObjRob.matrix[8]  = SDI_F->objectArray.objects[object_index].thetaMatRob.nz;    			  
  t3dObjRob.matrix[1]  = SDI_F->objectArray.objects[object_index].thetaMatRob.ox;
  t3dObjRob.matrix[5]  = SDI_F->objectArray.objects[object_index].thetaMatRob.oy;
  t3dObjRob.matrix[9]  = SDI_F->objectArray.objects[object_index].thetaMatRob.oz;
  t3dObjRob.matrix[2]  = SDI_F->objectArray.objects[object_index].thetaMatRob.ax;
  t3dObjRob.matrix[6]  = SDI_F->objectArray.objects[object_index].thetaMatRob.ay;
  t3dObjRob.matrix[10] = SDI_F->objectArray.objects[object_index].thetaMatRob.az;
  t3dObjRob.matrix[3]  = SDI_F->objectArray.objects[object_index].thetaMatRob.px;
  t3dObjRob.matrix[7]  = SDI_F->objectArray.objects[object_index].thetaMatRob.py;
  t3dObjRob.matrix[11] = SDI_F->objectArray.objects[object_index].thetaMatRob.pz; 

  t3dObjRob30.matrix[0]  = SDI_F->objectArray.objects[object_index].thetaMatRob30.nx;
  t3dObjRob30.matrix[4]  = SDI_F->objectArray.objects[object_index].thetaMatRob30.ny;
  t3dObjRob30.matrix[8]  = SDI_F->objectArray.objects[object_index].thetaMatRob30.nz;    			  
  t3dObjRob30.matrix[1]  = SDI_F->objectArray.objects[object_index].thetaMatRob30.ox;
  t3dObjRob30.matrix[5]  = SDI_F->objectArray.objects[object_index].thetaMatRob30.oy;
  t3dObjRob30.matrix[9]  = SDI_F->objectArray.objects[object_index].thetaMatRob30.oz;
  t3dObjRob30.matrix[2]  = SDI_F->objectArray.objects[object_index].thetaMatRob30.ax;
  t3dObjRob30.matrix[6]  = SDI_F->objectArray.objects[object_index].thetaMatRob30.ay;
  t3dObjRob30.matrix[10] = SDI_F->objectArray.objects[object_index].thetaMatRob30.az;
  t3dObjRob30.matrix[3]  = SDI_F->objectArray.objects[object_index].thetaMatRob30.px;
  t3dObjRob30.matrix[7]  = SDI_F->objectArray.objects[object_index].thetaMatRob30.py;
  t3dObjRob30.matrix[11] = SDI_F->objectArray.objects[object_index].thetaMatRob30.pz; 

  /* compute object pose in the robot frame*/
  t3dObjToOrigin =  t3dMainToOrigin * t3dObjRob;
  t3dObjToOrigin30 =  t3dMainToOrigin * t3dObjRob30;

  SDI_F->objectPublicArray.objects[object_index].thetaMatRobot.nx =  t3dObjRob.matrix[0];
  SDI_F->objectPublicArray.objects[object_index].thetaMatRobot.ny =  t3dObjRob.matrix[4];
  SDI_F->objectPublicArray.objects[object_index].thetaMatRobot.nz =  t3dObjRob.matrix[8];				  
  SDI_F->objectPublicArray.objects[object_index].thetaMatRobot.ox =  t3dObjRob.matrix[1];
  SDI_F->objectPublicArray.objects[object_index].thetaMatRobot.oy =  t3dObjRob.matrix[5];
  SDI_F->objectPublicArray.objects[object_index].thetaMatRobot.oz =  t3dObjRob.matrix[9];
  SDI_F->objectPublicArray.objects[object_index].thetaMatRobot.ax =  t3dObjRob.matrix[2];
  SDI_F->objectPublicArray.objects[object_index].thetaMatRobot.ay =  t3dObjRob.matrix[6];
  SDI_F->objectPublicArray.objects[object_index].thetaMatRobot.az =  t3dObjRob.matrix[10];
  SDI_F->objectPublicArray.objects[object_index].thetaMatRobot.px =  t3dObjRob.matrix[3]; 
  SDI_F->objectPublicArray.objects[object_index].thetaMatRobot.py =  t3dObjRob.matrix[7]; 
  SDI_F->objectPublicArray.objects[object_index].thetaMatRobot.pz =  t3dObjRob.matrix[11];

  t3dConvertTo(T3D_EULER, &t3dObjRob);
  
  SDI_F->objectPublicArray.objects[object_index].eulerRobot.yaw   =  t3dObjRob.euler.euler[0];
  SDI_F->objectPublicArray.objects[object_index].eulerRobot.pitch =  t3dObjRob.euler.euler[1];
  SDI_F->objectPublicArray.objects[object_index].eulerRobot.roll  =  t3dObjRob.euler.euler[2];
  SDI_F->objectPublicArray.objects[object_index].eulerRobot.x     =  t3dObjRob.euler.euler[3];
  SDI_F->objectPublicArray.objects[object_index].eulerRobot.y     =  t3dObjRob.euler.euler[4];
  SDI_F->objectPublicArray.objects[object_index].eulerRobot.z     =  t3dObjRob.euler.euler[5];
  
  SDI_F->objectPublicArray.objects[object_index].thetaMatOrigin.nx = t3dObjToOrigin.matrix[0];
  SDI_F->objectPublicArray.objects[object_index].thetaMatOrigin.ny = t3dObjToOrigin.matrix[4];
  SDI_F->objectPublicArray.objects[object_index].thetaMatOrigin.nz = t3dObjToOrigin.matrix[8];				  
  SDI_F->objectPublicArray.objects[object_index].thetaMatOrigin.ox = t3dObjToOrigin.matrix[1];
  SDI_F->objectPublicArray.objects[object_index].thetaMatOrigin.oy = t3dObjToOrigin.matrix[5];
  SDI_F->objectPublicArray.objects[object_index].thetaMatOrigin.oz = t3dObjToOrigin.matrix[9];
  SDI_F->objectPublicArray.objects[object_index].thetaMatOrigin.ax = t3dObjToOrigin.matrix[2];
  SDI_F->objectPublicArray.objects[object_index].thetaMatOrigin.ay = t3dObjToOrigin.matrix[6];
  SDI_F->objectPublicArray.objects[object_index].thetaMatOrigin.az = t3dObjToOrigin.matrix[10];
  //set correction in the origin frame, as these are the data used by spark
  SDI_F->objectPublicArray.objects[object_index].thetaMatOrigin.px = t3dObjToOrigin.matrix[3] + SDI_F->corOriginFrame.dx;
  SDI_F->objectPublicArray.objects[object_index].thetaMatOrigin.py = t3dObjToOrigin.matrix[7] + SDI_F->corOriginFrame.dy;
  SDI_F->objectPublicArray.objects[object_index].thetaMatOrigin.pz = t3dObjToOrigin.matrix[11]  + SDI_F->corOriginFrame.dz;  

  t3dConvertTo(T3D_EULER, &t3dObjToOrigin);
  t3dConvertTo(T3D_EULER, &t3dObjToOrigin30);
  
  SDI_F->objectPublicArray.objects[object_index].eulerOrigin.yaw   =   t3dObjToOrigin.euler.euler[0];
  SDI_F->objectPublicArray.objects[object_index].eulerOrigin.pitch =   t3dObjToOrigin.euler.euler[1];
  SDI_F->objectPublicArray.objects[object_index].eulerOrigin.roll  =   t3dObjToOrigin.euler.euler[2];
  SDI_F->objectPublicArray.objects[object_index].eulerOrigin.x     =   t3dObjToOrigin.euler.euler[3];
  SDI_F->objectPublicArray.objects[object_index].eulerOrigin.y     =   t3dObjToOrigin.euler.euler[4];
  SDI_F->objectPublicArray.objects[object_index].eulerOrigin.z     =   t3dObjToOrigin.euler.euler[5];

  SDI_F->objectPublicArray.objects[object_index].eulerOrigin30.yaw   =   t3dObjToOrigin30.euler.euler[0];
  SDI_F->objectPublicArray.objects[object_index].eulerOrigin30.pitch =   t3dObjToOrigin30.euler.euler[1];
  SDI_F->objectPublicArray.objects[object_index].eulerOrigin30.roll  =   t3dObjToOrigin30.euler.euler[2];
  SDI_F->objectPublicArray.objects[object_index].eulerOrigin30.x     =   t3dObjToOrigin30.euler.euler[3];
  SDI_F->objectPublicArray.objects[object_index].eulerOrigin30.y     =   t3dObjToOrigin30.euler.euler[4];
  SDI_F->objectPublicArray.objects[object_index].eulerOrigin30.z     =   t3dObjToOrigin30.euler.euler[5];
  
  return 1;
}



//Deallocate all the global dynamic variables.
void vimanResetAll()
{
  SDI_F->vimanInitIsDone= VIMAN_DISABLE;
  SDI_F->flagDisplay= VIMAN_DISABLE;
  SDI_F->flagTracking= VIMAN_DISABLE;
  SDI_F->objectArray.nbObjects= 0;

  if(CAPTURE_L!=NULL) 
  {  
    cvReleaseCapture(&CAPTURE_L); 
    CAPTURE_L= NULL;
  }printf("CAPTURE_L released\n");  

  switch(SDI_F->initStruct.inputStream) {
  case VIMAN_INPUT_WEBCAM:
   IMAGE_SRC_L= NULL;
   break;
  }
  if(IMAGE_SRC_L!=NULL)  
  {
    cvReleaseImage(&IMAGE_SRC_L); 
    IMAGE_SRC_L= NULL;
  }printf("IMAGE_SRC_L released\n");  

  if(IMAGE_GRAY_L!=NULL)  
  {
    cvReleaseImage(&IMAGE_GRAY_L); 
    IMAGE_GRAY_L= NULL;
  }printf("IMAGE_GRAY_L released\n");  

  if(IMAGE_GRAY_EQUALIZED_L!=NULL)  
  {
    cvReleaseImage(&IMAGE_GRAY_EQUALIZED_L); 
    IMAGE_GRAY_EQUALIZED_L= NULL;
  }printf("IMAGE_GRAY_EQUALIZED_L released\n");  

  if(IMAGE_UNDIST_L!=NULL) 
  {
    cvReleaseImage(&IMAGE_UNDIST_L); 
    IMAGE_UNDIST_L= NULL;
  }printf("IMAGE_UNDIST_L released\n");  


  if(IMAGE_MOD_L!=NULL)   
  {
    cvReleaseImage(&IMAGE_MOD_L); 
    IMAGE_MOD_L= NULL;
  }printf("IMAGE_MOD_L released\n");  


  if(INTRINSIC_MATRIX_L!=NULL)   
  {
    cvReleaseMat(&INTRINSIC_MATRIX_L); 
    INTRINSIC_MATRIX_L= NULL;
  }printf("INTRINSIC_MATRIX_L released\n");  


  if(DIST_COEFFS_L!=NULL)   
  {
    cvReleaseMat(&DIST_COEFFS_L); 
    DIST_COEFFS_L= NULL;
  }printf("DIST_COEFFS_L released\n");  


  if(TRACKER_L!=NULL)
  {
    delete TRACKER_L;
    TRACKER_L= NULL;
  }printf("TRACKER_L released\n");  


  if(CAMERA_BUFFER_L!=NULL)
  {
    delete [] CAMERA_BUFFER_L;
    CAMERA_BUFFER_L= NULL;
  }printf("CAMERA_BUFFER_L released\n");  





  if(CAPTURE_R!=NULL) 
  {  
    cvReleaseCapture(&CAPTURE_R); 
    CAPTURE_R= NULL;
  }printf("CAPTURE_R released\n");  

  if(IMAGE_SRC_R!=NULL)  
  {
    cvReleaseImage(&IMAGE_SRC_R); 
    IMAGE_SRC_R= NULL;
  }printf("IMAGE_SRC_R released\n");  

  if(IMAGE_GRAY_R!=NULL)  
  {
    cvReleaseImage(&IMAGE_GRAY_R); 
    IMAGE_GRAY_R= NULL;
  }printf("IMAGE_GRAY_R released\n");  

  if(IMAGE_GRAY_EQUALIZED_R!=NULL)  
  {
    cvReleaseImage(&IMAGE_GRAY_EQUALIZED_R); 
    IMAGE_GRAY_EQUALIZED_R= NULL;
  }printf("IMAGE_GRAY_EQUALIZED_R released\n");  

  if(IMAGE_UNDIST_R!=NULL) 
  {
    cvReleaseImage(&IMAGE_UNDIST_R); 
    IMAGE_UNDIST_R= NULL;
  }printf("IMAGE_UNDIST_R released\n");  


  if(IMAGE_MOD_R!=NULL)   
  {
    cvReleaseImage(&IMAGE_MOD_R); 
    IMAGE_MOD_R= NULL;
  }printf("IMAGE_MOD_R released\n");  


  if(INTRINSIC_MATRIX_R!=NULL)   
  {
    cvReleaseMat(&INTRINSIC_MATRIX_R); 
    INTRINSIC_MATRIX_R= NULL;
  }printf("INTRINSIC_MATRIX_R released\n");  


  if(DIST_COEFFS_R!=NULL)   
  {
    cvReleaseMat(&DIST_COEFFS_R); 
    DIST_COEFFS_R= NULL;
  }printf("DIST_COEFFS_R released\n");  


  if(TRACKER_R!=NULL)
  {
    delete TRACKER_R;
    TRACKER_R= NULL;
  }printf("TRACKER_R released\n");  


  if(CAMERA_BUFFER_R!=NULL)
  {
    delete [] CAMERA_BUFFER_R;
    CAMERA_BUFFER_R= NULL;
  }printf("CAMERA_BUFFER_R released\n");  


  for(int u=VIMAN_MAX_OBJ_NB-1; u>=0; u--) {
    if(TRANSLATION_VECTOR_L[u]!=NULL) {
      cvReleaseMat(&(TRANSLATION_VECTOR_L[u]));
      TRANSLATION_VECTOR_L[u] = NULL;
    }
    if(ROTATION_VECTOR_L[u]!=NULL) {
      cvReleaseMat(&(ROTATION_VECTOR_L[u]));
      ROTATION_VECTOR_L[u] = NULL;
    }
    if(ROTATION_VECTOR_MARKER_L[u]!=NULL) {
      cvReleaseMat(&(ROTATION_VECTOR_MARKER_L[u]));
      ROTATION_VECTOR_MARKER_L[u] = NULL;
    }
    if(TRANSLATION_VECTOR_R[u]!=NULL) {
      cvReleaseMat(&(TRANSLATION_VECTOR_R[u]));
      TRANSLATION_VECTOR_R[u] = NULL;
    }
    if(ROTATION_VECTOR_R[u]!=NULL) {
      cvReleaseMat(&(ROTATION_VECTOR_R[u]));
      ROTATION_VECTOR_R[u] = NULL;
    }
  }
  printf("TRANSLATION_VECTOR_L[] released\n");  
  printf("ROTATION_VECTOR_L[] released\n");  
  printf("ROTATION_VECTOR_MARKER_L[] released\n");  
  printf("TRANSLATION_VECTOR_R[] released\n");  
  printf("ROTATION_VECTOR_R[] released\n");  
}

// Compute a visibility criterion of the image of a marker (corner positions on the image).
// In case an object has several markers, it allows to choose the most visible to estimate the object pose,
// to have a more precise estimation.
// A marker has a good visibility if it is big in the image and the most perpendicular possible with respect
// to the image plane (the most square-shaped).
// The criterion is a the product of the marker area and the inverse of the maximum difference between edge lengths.
double vimanComputeMarkerVisibility(CvPoint2D32f marker_corners[4])
{
   double a, b, c, s, area;
   double min_length, max_length;

   //the marker image area is the sum of two triangles areas:
   a= sqrt( pow(marker_corners[1].x - marker_corners[0].x, 2) + pow(marker_corners[1].y - marker_corners[0].y, 2) );
   b= sqrt( pow(marker_corners[2].x - marker_corners[1].x, 2) + pow(marker_corners[2].y - marker_corners[1].y, 2) );
   c= sqrt( pow(marker_corners[0].x - marker_corners[2].x, 2) + pow(marker_corners[0].y - marker_corners[2].y, 2) );
   s= ( a + b + c )/2;

   area= ( s*(s-a)*(s-b)*(s-c) ); 

   min_length= max_length= a;
   if(b < min_length) min_length= b;
   if(b > max_length) max_length= b;


   a= sqrt( pow(marker_corners[3].x - marker_corners[2].x, 2) + pow(marker_corners[3].y - marker_corners[2].y, 2) );
   b= sqrt( pow(marker_corners[0].x - marker_corners[3].x, 2) + pow(marker_corners[0].y - marker_corners[3].y, 2) );
   s= ( a + b + c )/2;

   area+= ( s*(s-a)*(s-b)*(s-c) ); 

   if(a < min_length) min_length= a;
   if(a > max_length) max_length= a;
   if(b < min_length) min_length= b;
   if(b > max_length) max_length= b;

   return ( area/fabs(max_length-min_length) );
}

//void unproject(CvMat intrinsic_matrix, const CvMat* image_point, double depth, const CvMat* object_point)
int unprojectPoint( CvMat *intrinsic_matrix,  CvPoint2D32f* image_point, double depth, CvPoint3D32f* object_point)
{
 if( !CV_IS_MAT(intrinsic_matrix) || intrinsic_matrix->rows != 3 || intrinsic_matrix->cols != 3  ||
     (CV_MAT_TYPE(intrinsic_matrix->type) != CV_32FC1 && CV_MAT_TYPE(intrinsic_matrix->type) != CV_64FC1) )
 {
   printf("unprojectPoint: Intrinsic matrix must be a valid 3x3 floating-point matrix" );
   return 0;
 }

 object_point->x= depth*( image_point->x - cvmGet(intrinsic_matrix, 0, 2) )/cvmGet(intrinsic_matrix, 0, 0);
 object_point->y= depth*( image_point->y - cvmGet(intrinsic_matrix, 1, 2) )/cvmGet(intrinsic_matrix, 1, 1);
 object_point->z= depth;

 return 1;
}


bool LineLineIntersect(double p1[3], double p2[3], double p3[3], double p4[3], double pa[3], double pb[3], double *mua, double *mub)
{
  int i;
  double p13[3], p43[3], p21[3];
  double d1343, d4321, d1321, d4343, d2121;
  double numer, denom;
  double eps= 1e-7;

  for(i=0; i<3; i++)
  {
    p13[i] = p1[i] - p3[i];
    p43[i] = p4[i] - p3[i];
  }


  // if (fabs(p43[0])  < eps && fabs(p43[1])  < eps && fabs(p43[2])  < EPS)
  //    return false;

  for(i=0; i<3; i++)
  {
    p21[i] = p2[i] - p1[i];
  }

  // if (fabs(p21[0])  < eps && fabs(p21[1])  < eps && fabs(p21[2])  < EPS)
  //   return false;

  d1343 = p13[0] * p43[0] + p13[1] * p43[1] + p13[2] * p43[2];
  d4321 = p43[0] * p21[0] + p43[1] * p21[1] + p43[2] * p21[2];
  d1321 = p13[0] * p21[0] + p13[1] * p21[1] + p13[2] * p21[2];
  d4343 = p43[0] * p43[0] + p43[1] * p43[1] + p43[2] * p43[2];
  d2121 = p21[0] * p21[0] + p21[1] * p21[1] + p21[2] * p21[2];

  denom = d2121 * d4343 - d4321 * d4321;

  // if (fabs(denom) < eps)
  //   return false;

  numer = d1343 * d4321 - d1321 * d4343;

  *mua = numer / denom;
  *mub = (d1343 + d4321 * (*mua)) / d4343;

  for(i=0; i<3; i++)
  {
    pa[i] = p1[i] + (*mua) * p21[i];
    pb[i] = p3[i] + (*mub) * p43[i];
  }

  return true;
}

/* vimanSetCameraLeftPoseMain  -  codel EXEC of SetCameraLeftPose
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
vimanSetCameraLeftPoseMain(VimanThetaMat *CAMERA_LEFT_ROB_POSE, int *report)
{
  /* ... add your code here ... */
  return ETHER;
}

